import pandas as pd

from accelera.src.custom.transformer import CustomTransformer


class IQRTransform(CustomTransformer):
    def __init__(self, info, cols):
        self.info = info
        self.cols = cols

    def fit(self, X, y=None):
        return self

    def transform(self, X):
        X_copy = X.copy()
        for i, col in enumerate(self.cols):
            if self.info[col]["col_type"] in ["numerical", "continuous"]:
                lower, upper = self.info[col]["outliers_info"]
                if lower is not None and upper is not None:
                    if isinstance(X_copy, pd.DataFrame):
                        X_copy[col] = X_copy[col].clip(lower, upper)
                    else:
                        X_copy[:, i] = X_copy[:, i].clip(lower, upper)
        return X_copy

    def get_feature_names_out(self, input_features=None):
        return input_features



from accelera.src.accelera_pipe.core.metric_display import MetricDisplay


class DisplayString(MetricDisplay):
    def __init__(self, metric_name, values):
        super().__init__(metric_name, values)

    def execute(self):
        content = f"<div>\n<h3>Metric name: {self.metric_name}</h3>\n"
        for value in self.values:
            new_content = (
                "<div>\n"
                "<h4>\n"
                f"Metric id: {value['metric id']}</h4>\n"
                f"<pre>{value['result'].strip()}</pre>\n"
                "</div>\n"
            )
            content = content + new_content
        content = content + "</div>\n"
        return content



import os
import shutil
import tempfile

import numpy as np
import pandas as pd
import pytest
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import LabelEncoder

from accelera.src.automl.core.text_training_preprocessing import (
    TextTrainingPreprocessing,
)
from accelera.src.utils.preprocessing import check_path_exists


class TestTextTrainingPreprocessing:
    @pytest.fixture(autouse=True)
    def temp_folder(self):
        self.temp_dir = tempfile.mkdtemp()
        yield self.temp_dir
        shutil.rmtree(self.temp_dir)

    @pytest.fixture(autouse=True)
    def sample_data(self):
        self.df = pd.DataFrame(
            {
                "text": [
                    "Hello ! how are you?",
                    "Welcome",
                    "Welcome",
                    "She's great",
                    "it is terrible",
                    "great terrible",
                    "terrible great welcome",
                    "Hello",
                    "Hello welcome",
                    None,
                ],
                "class": [
                    "pos",
                    "pos",
                    "pos",
                    "pos",
                    None,
                    "neg",
                    "pos",
                    "neg",
                    "pos",
                    None,
                ],
                "wrong_text": [1.1, 2, 3, 5, 5, 5, 2, 3, 4, 5],
            }
        )

    def test_text_error_checks(self):
        with pytest.raises(ValueError):
            TextTrainingPreprocessing(
                df=None,
                target_col="class",
                text_col="text",
                folder_path=self.temp_dir,
            )
        with pytest.raises(ValueError):
            TextTrainingPreprocessing(
                df=self.df,
                target_col="class",
                text_col="text",
                folder_path=None,
            )

        with pytest.raises(ValueError):
            TextTrainingPreprocessing(
                df=self.df,
                target_col="unvalid",
                text_col="text",
                folder_path=self.temp_dir,
            )
        with pytest.raises(ValueError):
            TextTrainingPreprocessing(
                df=self.df,
                target_col="text",
                text_col="unvalid",
                folder_path=self.temp_dir,
            )
        with pytest.raises(ValueError):
            TextTrainingPreprocessing(
                df=self.df,
                target_col="text",
                text_col="wrong_text",
                folder_path=self.temp_dir,
            )
        with pytest.raises(ValueError):
            TextTrainingPreprocessing(
                df=self.df,
                target_col="wrong_text",
                text_col="class",
                folder_path=self.temp_dir,
            )
        with pytest.raises(ValueError):
            TextTrainingPreprocessing(
                df=self.df,
                target_col="class",
                text_col="class",
                folder_path=self.temp_dir,
            )

    def test_text_setup(self):
        tp = TextTrainingPreprocessing(self.df, "class", "text", self.temp_dir)
        expected_data = self.df[["text", "class"]].copy()
        assert tp.info["target_col"] == "class"
        assert tp.info["text_col"] == "text"
        pd.testing.assert_frame_equal(tp.df, expected_data)

    def test_text_data_overview(self):
        self.df = self.df.drop(columns=["wrong_text"])
        tp = TextTrainingPreprocessing(
            df=self.df,
            target_col="class",
            text_col="text",
            folder_path=self.temp_dir,
        )
        tp.data_overview()
        assert tp.report_data is not None
        assert "data_overview" in tp.report_data
        data_overview = tp.report_data["data_overview"]
        assert data_overview["shape"] == (10, 2)
        assert data_overview["duplicates_sum"] == 1
        assert data_overview["duplicates_percentage"] == float(1 / 10) * 100
        assert data_overview["numerical_describe"] is None
        assert data_overview["categorical_describe"] is not None
        assert data_overview["data_head"].shape == (5, 2)
        assert data_overview["lower_data_head"].shape == (5, 2)
        assert data_overview["missing_values"].equals(self.df.isnull().sum())

    def test_text_drop_dupplicates(self):
        tp = TextTrainingPreprocessing(
            df=self.df,
            target_col="class",
            text_col="text",
            folder_path=self.temp_dir,
        )
        shape_before_drop = tp.df.shape
        tp.data_overview()
        tp.drop_duplicates()
        shape_after_drop = tp.df.shape
        assert shape_after_drop[0] == shape_before_drop[0] - 1
        assert shape_after_drop[1] == shape_before_drop[1]
        assert tp.report_data["drop_duplicates"]["shape"] == shape_after_drop
        assert self.df.duplicated().sum() == 0
        assert tp.report_data["drop_duplicates"]["duplicates_sum"] == 0
        assert tp.report_data["drop_duplicates"]["duplicates_percentage"] == 0

    def test_text_split_data(self):
        self.df = self.df.drop(columns=["wrong_text"])

        tp = TextTrainingPreprocessing(
            df=self.df,
            target_col="class",
            text_col="text",
            folder_path=self.temp_dir,
        )
        tp.data_overview()
        tp.drop_duplicates()
        X_train_manual, X_val_manual, y_train_manual, y_val_manual = (
            train_test_split(
                tp.df.drop(columns=["class"]),
                tp.df["class"],
                test_size=0.2,
                random_state=42,
            )
        )
        X_train, X_val, y_train, y_val = tp.split_data()
        assert X_train.shape[0] == X_train_manual.shape[0]
        assert X_val.shape[0] == X_val_manual.shape[0]
        assert y_train.shape[0] == y_train_manual.shape[0]
        assert y_val.shape[0] == y_val_manual.shape[0]
        assert tp.report_data["split"]["X_train_shape"] == X_train.shape
        assert tp.report_data["split"]["X_val_shape"] == X_val.shape
        assert tp.report_data["split"]["y_train_shape"] == y_train.shape
        assert tp.report_data["split"]["y_val_shape"] == y_val.shape
        assert X_train.equals(X_train_manual)
        assert X_val.equals(X_val_manual)
        assert y_train.equals(y_train_manual)
        assert y_val.equals(y_val_manual)

    def test_text_make_graphs(self):
        tp = TextTrainingPreprocessing(
            df=self.df,
            target_col="class",
            text_col="text",
            folder_path=self.temp_dir,
        )
        tp.data_overview()
        tp.drop_duplicates()
        X_train, X_val, y_train, y_val = tp.split_data()
        tp.make_graphs(X_train, y_train)
        graphs_path = os.path.join(self.temp_dir, "graphs")
        assert check_path_exists(graphs_path, "")
        graphs_files = os.listdir(graphs_path)
        expected_num_graphs = 2
        assert len(graphs_files) == expected_num_graphs
        assert "class.png" in graphs_files
        assert "text.png" in graphs_files

    def test_text_custom_text_tokenizer(self):
        tp = TextTrainingPreprocessing(
            df=self.df,
            target_col="class",
            text_col="text",
            folder_path=self.temp_dir,
        )
        text = "Hello ? he's --  can’t won't 'no' history like123nice "
        tokens = tp.custom_text_tokenizer(text)
        tokens = " ".join(tokens)
        assert tokens == "hello not not no histori like 123 nice"

    def test_text_features_preprocessing(self):
        tp = TextTrainingPreprocessing(
            df=self.df,
            target_col="class",
            text_col="text",
            folder_path=self.temp_dir,
        )
        tp.data_overview()
        tp.drop_duplicates()
        tp.report_data["preprocessing"] = []
        X_train, X_val, y_train, y_val = tp.split_data()
        X_train_filled = X_train["text"].fillna("")
        X_val_filled = X_val["text"].fillna("")
        X_train_processed, X_val_processed = tp.features_preprocessing(
            X_train, X_val
        )
        training_preprocessor = TfidfVectorizer(
            max_features=5000,
            ngram_range=(1, 2),
            max_df=0.85,
            min_df=3,
            tokenizer=tp.custom_text_tokenizer,
            token_pattern=None,
            lowercase=False,
        )

        X_train_processed_manual = training_preprocessor.fit_transform(
            X_train_filled
        )
        X_val_processed_manual = training_preprocessor.transform(X_val_filled)
        assert X_train_processed_manual.shape == X_train_processed.shape
        assert np.allclose(
            X_train_processed_manual.toarray(), X_train_processed.toarray()
        )
        assert np.allclose(
            X_val_processed_manual.toarray(), X_val_processed.toarray()
        )
        assert check_path_exists(self.temp_dir, "training_preprocessor.pkl")

    def test_text_target_preprocessing(self):
        tp = TextTrainingPreprocessing(
            df=self.df,
            target_col="class",
            text_col="text",
            folder_path=self.temp_dir,
        )
        tp.data_overview()
        tp.drop_duplicates()
        tp.report_data["after_preprocessing"] = {}
        tp.report_data["preprocessing"] = []
        X_train, X_val, y_train, y_val = tp.split_data()
        mode = y_train.mode()[0]
        encoder = LabelEncoder()
        y_train_fill = y_train.fillna(mode)
        y_val_fill = y_val.fillna(mode)
        y_train_manual = encoder.fit_transform(y_train_fill)
        y_val_manual = encoder.transform(y_val_fill)
        y_train_processed, y_val_processed = tp.target_preprocessing(y_train, y_val)
        assert tp.info["target_mode"] == mode
        assert np.allclose(y_val_manual, y_val_processed)
        assert np.allclose(y_train_manual, y_train_processed)
        assert check_path_exists(self.temp_dir, "target_preprocessor.pkl")
        assert check_path_exists(self.temp_dir, "data_info.pkl")



import os

import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns

from accelera.src.automl.wrappers.graph_base import GraphBase


class ImageLabelClassification(GraphBase):
    def __init__(
        self,
        labels,
        label2class_mapping,
        invalid_labels,
        folder_path,
        title="",
        file_name="",
    ):
        super().__init__(folder_path)
        label2class_mapping = label2class_mapping
        self.valid_df = pd.DataFrame({"labels": labels})
        self.valid_df["labels"] = self.valid_df["labels"].map(label2class_mapping)
        self.total_df = None
        self.invalid_df = None
        self.title = title
        self.file_name = file_name
        if invalid_labels is not None and len(invalid_labels) > 0:
            self.invalid_df = pd.DataFrame({"labels": invalid_labels})
            self.invalid_df["labels"] = self.invalid_df["labels"].map(
                label2class_mapping
            )
            self.total_df = pd.concat(
                [self.valid_df, self.invalid_df], axis=0, ignore_index=True
            )
            self.length = [0, 0]
            self.length[0] = len(self.valid_df["labels"]) / len(
                self.total_df["labels"]
            )
            self.length[1] = len(self.invalid_df["labels"]) / len(
                self.total_df["labels"]
            )
        else:
            self.total_df = self.valid_df
            self.length = [1, 0]

    def build_graph(self):
        if self.invalid_df is not None:
            fig, ax = plt.subplots(1, 4, figsize=(16, 5))
        else:
            fig, ax = plt.subplots(1, 3, figsize=(12, 5))

        fig.suptitle(self.title, fontsize=20)
        sns.countplot(data=self.total_df, x="labels", ax=ax[0])
        ax[0].set_title("Total images classes distribution")
        ax[0].set_xlabel("Classes")
        ax[0].set_ylabel("Count")
        ax[1].pie(
            [float(self.length[0] * 100), float(100 * self.length[1])],
            labels=["valid images", "invalid images"],
            autopct="%1.1f%%",
            colors=["#021D25", "#ADD8E6"],
        )
        ax[1].set_title("Valid vs Invalid percentage")
        sns.countplot(data=self.valid_df, x="labels", ax=ax[2])
        ax[2].set_title("Valid images classes distribution")
        ax[2].set_xlabel("Classes")
        ax[2].set_ylabel("Count")
        if self.invalid_df is not None:
            sns.countplot(data=self.invalid_df, x="labels", ax=ax[3])
            ax[3].set_title("Invalid images classes distribution")
            ax[3].set_xlabel("Classes")
            ax[3].set_ylabel("Count")
        plt.tight_layout()
        plt.savefig(os.path.join(self.folder_path, f"{self.file_name}.png"))
        plt.close()



from accelera.src.automl.core.tabular_preprocessing_base import (
    TabularPreprocessingBase,
)
from accelera.src.utils.preprocessing import check_path_exists
from accelera.src.utils.preprocessing import load_pickle


class TestingTabularPreprocessingBase(TabularPreprocessingBase):
    def __init__(self, df, folder_path=None):
        super().__init__(df, folder_path)
        check_path_exists(self.folder_path, "target_preprocessor.pkl")
        check_path_exists(self.folder_path, "training_preprocessor.pkl")
        self.target_preprocessor = load_pickle(
            self.folder_path, "target_preprocessor.pkl"
        )
        self.training_preprocessor = load_pickle(
            self.folder_path, "training_preprocessor.pkl"
        )
        if self.target_preprocessor is None:
            raise ValueError(
                "target_preprocessor cannot be None please run training "
                "preprocessing first to create the target preprocessor"
            )

        if self.training_preprocessor is None:
            raise ValueError(
                "training_preprocessor cannot be None please run training "
                "preprocessing first to create the training preprocessor"
            )
        self.features_only = False



import logging
from unittest.mock import MagicMock
from unittest.mock import patch

from accelera.src.utils.accelera_utils import get_correct_metric_class
from accelera.src.utils.accelera_utils import get_instance_type
from accelera.src.utils.accelera_utils import get_metric_object
from accelera.src.utils.accelera_utils import print_msg


class TestPrintMsg:
    def test_print_msg_interactive_with_line_break(self, capsys, monkeypatch):
        monkeypatch.setitem(print_msg.__globals__, "interactive", True)
        print_msg("Hello Accelera!")
        captured = capsys.readouterr()
        assert captured.out == "Hello Accelera!\n"

    def test_print_msg_interactive_without_line_break(self, capsys, monkeypatch):
        monkeypatch.setitem(print_msg.__globals__, "interactive", True)
        print_msg("NoBreak", line_break=False)
        captured = capsys.readouterr()
        assert captured.out == "NoBreak"

    def test_print_msg_logging_mode(self, caplog, capsys, monkeypatch):
        monkeypatch.setitem(print_msg.__globals__, "interactive", False)
        with caplog.at_level(logging.INFO):
            print_msg("Logged Info", level="info")
        assert "Logged Info" in caplog.text
        captured = capsys.readouterr()
        assert captured.out == ""


class TestGetMetricObject:
    def test_get_existing_metric(self):
        with patch("accelera.src.utils.accelera_utils.metrics") as mock_metrics:
            mock_metrics.accuracy = "mock_accuracy_function"
            result = get_metric_object("accuracy")
            assert result == "mock_accuracy_function"

    def test_get_nonexistent_metric(self):
        with patch("accelera.src.utils.accelera_utils.metrics") as mock_metrics:
            mock_metrics.nonexistent_metric = None
            result = get_metric_object("nonexistent_metric")
            assert result is None

    def test_get_empty_metric_name(self):
        result = get_metric_object("")
        assert result is None


class TestGetCorrectMetricClass:
    def test_supervised_metric_detection(self):
        def supervised_metric(y_true, y_pred):
            return 0

        class DummySupervisedWrapper:
            def __init__(
                self,
                metric_name,
                metric,
                y_true,
                plot_func,
                labels_name,
                headers_name,
                **params,
            ):
                self.metric_name = metric_name
                self.metric = metric
                self.y_true = y_true
                self.plot_func = plot_func
                self.params = params
                self.labels_name = labels_name
                self.headers_name = headers_name

        with patch(
            "accelera.src.utils.accelera_utils.SupervisedMetric",
            DummySupervisedWrapper,
        ):
            result = get_correct_metric_class(
                "test_metric", supervised_metric, y_true=[0, 1, 1, 0]
            )
            assert isinstance(result, DummySupervisedWrapper)
            assert result.metric_name == "test_metric"

    def test_unsupervised_metric_detection(self):
        def unsupervised_metric(X, labels):
            return 0

        class DummyUnSupervisedWrapper:
            def __init__(
                self,
                metric_name,
                metric,
                plot_func,
                labels_name,
                headers_name,
                **params,
            ):
                self.metric_name = metric_name
                self.metric = metric
                self.params = params
                self.plot_func = plot_func
                self.labels_name = labels_name
                self.headers_name = headers_name

        with patch(
            "accelera.src.utils.accelera_utils.UnSupervisedMetric",
            DummyUnSupervisedWrapper,
        ):
            result = get_correct_metric_class(
                "test_metric", unsupervised_metric, X=[[1], [2], [3]]
            )
            assert isinstance(result, DummyUnSupervisedWrapper)
            assert result.metric_name == "test_metric"

    def test_invalid_metric_returns_none(self):
        def invalid_metric(a, b, c):
            return 0

        result = get_correct_metric_class("test_metric", invalid_metric)
        assert result is None


class TestGetInstanceType:
    def test_supervised_algorithms(self):
        clf = MagicMock()
        clf.__class__.__name__ = "RandomForestClassifier"
        clf.__class__.__module__ = "sklearn.ensemble"
        assert get_instance_type(clf) == "supervised"

        reg = MagicMock()
        reg.__class__.__name__ = "LinearRegression"
        reg.__class__.__module__ = "sklearn.linear_model"
        assert get_instance_type(reg) == "supervised"

    def test_unsupervised_algorithms(self):
        kmeans = MagicMock()
        kmeans.__class__.__name__ = "KMeans"
        kmeans.__class__.__module__ = "sklearn.cluster"
        assert get_instance_type(kmeans) == "unsupervised"

        pca = MagicMock()
        pca.__class__.__name__ = "PCA"
        pca.__class__.__module__ = "sklearn.decomposition"
        assert get_instance_type(pca) == "unsupervised"

    def test_feature_selection_supervised(self):
        selector = MagicMock()
        selector.__class__.__name__ = "SelectKBest"
        selector.__class__.__module__ = "sklearn.feature_selection"
        assert get_instance_type(selector) == "supervised"

    def test_preprocessing_unsupervised(self):
        scaler = MagicMock()
        scaler.__class__.__name__ = "StandardScaler"
        scaler.__class__.__module__ = "sklearn.preprocessing"
        assert get_instance_type(scaler) == "unsupervised"

    def test_real_sklearn_objects(self):
        try:
            from sklearn.cluster import KMeans
            from sklearn.ensemble import RandomForestClassifier
            from sklearn.feature_selection import SelectKBest
            from sklearn.preprocessing import StandardScaler

            assert get_instance_type(RandomForestClassifier()) == "supervised"
            assert get_instance_type(KMeans(n_clusters=2)) == "unsupervised"
            assert get_instance_type(SelectKBest()) == "supervised"
            assert get_instance_type(StandardScaler()) == "unsupervised"
        except ImportError:
            pass  # Skip if sklearn not available

    def test_unknown_defaults_to_supervised(self):
        class Unknown:
            pass

        assert get_instance_type(Unknown()) == "supervised"



import pandas as pd

from accelera.src.custom.transformer import CustomTransformer


class FrequencyEncoderTransform(CustomTransformer):
    def __init__(self):
        self.mapping_ = {}
        self.cols_ = None

    def fit(self, X, y=None):
        self.mapping_ = {}
        self.cols_ = None
        if not isinstance(X, pd.DataFrame):
            X = pd.DataFrame(X)

        self.cols_ = X.columns
        for col in self.cols_:
            self.mapping_[col] = X[col].value_counts(normalize=True).to_dict()
        return self

    def transform(self, X):
        if not isinstance(X, pd.DataFrame):
            X = pd.DataFrame(X, columns=self.cols_)

        X_copy = X.copy()
        for col in self.cols_:
            X_copy[col] = X_copy[col].map(self.mapping_[col]).fillna(0)
        return X_copy.values

    def get_feature_names_out(self, input_features=None):
        return input_features



from accelera.src.automl.core.testing_tabular_preprocessing_base import (
    TestingTabularPreprocessingBase,
)
from accelera.src.utils.preprocessing import check_path_exists
from accelera.src.utils.preprocessing import drop_columns
from accelera.src.utils.preprocessing import load_pickle
from accelera.src.utils.preprocessing import lower_data


class ClassicalTestingPreprocessing(TestingTabularPreprocessingBase):
    def __init__(self, df, folder_path=None):
        super().__init__(df, folder_path=folder_path)
        check_path_exists(self.folder_path, "data_columns.pkl")
        check_path_exists(self.folder_path, "col_drop.pkl")
        check_path_exists(self.folder_path, "target_info.pkl")
        self.data_columns = load_pickle(self.folder_path, "data_columns.pkl")
        self.col_drop = load_pickle(self.folder_path, "col_drop.pkl")
        self.target_info = load_pickle(self.folder_path, "target_info.pkl")

        if self.target_info is None:
            raise ValueError(
                "target_info cannot be None please run training "
                "preprocessing first to create the target info"
            )
        if self.target_info.get("col_name") is None:
            raise ValueError(
                "target_info must contain 'col_name' key with the target "
                "column run training preprocessing first to create the "
                "target info"
            )
        if self.target_info["col_name"] not in self.data_columns:
            raise ValueError(
                "target column specified in target_info not found in "
                "data_columns run training preprocessing first to create "
                "the correct data columns"
            )
        if self.target_info.get("problem_type") is None:
            raise ValueError(
                "target_info must contain 'problem_type' key with the "
                "target problem type run training preprocessing first to "
                "create the target info"
            )
        if self.target_info["problem_type"] not in [
            "classification",
            "regression",
        ]:
            raise ValueError(
                "problem_type in target_info must be either "
                "'classification' or 'regression'"
            )

        if self.target_info["col_name"] not in self.df.columns:
            self.features_only = True
            self.data_columns.remove(self.target_info["col_name"])
        if not self.data_columns == list(self.df.columns):
            raise ValueError(
                "training data columns do not match the testing data columns"
            )
        lower_data(self.df)
        if self.features_only:
            self.X_test = self.df
            self.y_test = None
        else:
            self.X_test = self.df.drop(columns=[self.target_info["col_name"]])
            self.y_test = self.df[self.target_info["col_name"]]

    def target_preprocessing(self):
        try:
            if self.target_info["problem_type"] == "classification":
                self.y_test.fillna(self.target_info["mode"], inplace=True)
                self.y_test = self.target_preprocessor.transform(self.y_test)
            elif self.target_info["problem_type"] == "regression":
                self.y_test.fillna(self.target_info["median"], inplace=True)
                self.y_test = self.target_preprocessor.transform(
                    self.y_test.values.reshape(-1, 1)
                ).ravel()
        except Exception as e:
            raise ValueError(f"Error in target preprocessing: {e}")

    def common_preprocessing(self):
        try:
            drop_columns(self.X_test, self.col_drop)
            self.X_test = self.training_preprocessor.transform(self.X_test)
            if not self.features_only:
                self.target_preprocessing()
            return self.X_test, self.y_test
        except Exception as e:
            raise ValueError(f"Error in common preprocessing: {e}")



try:
    import graph
except ImportError as e:
    raise ImportError(
        "The 'graph' C++ module could not be imported. "
        "Please ensure it is built and available in your PYTHONPATH."
    ) from e


class PipelineBase:
    def __init__(self, _graph=None):
        self.__graph = _graph if _graph is not None else graph.Graph()
        self.__graph.enableParallelExecution(True)
        self.types = {
            "preprocess": graph.NodeType.PREPROCESS,
            "model": graph.NodeType.MODEL,
            "predict": graph.NodeType.PREDICT,
            "metric": graph.NodeType.METRIC,
            "merge": graph.NodeType.MERGE,
        }

    def __call__(self, *args, **kwargs):
        raise NotImplementedError(
            "__call__ method must be implemented in subclasses."
        )

    def set_multicore_threshold(self, threshold):
        self.__graph.setMulticoreThreshold(threshold)
        return self

    def disable_parallel_execution(self):
        self.__graph.enableParallelExecution(False)
        return self

    def save_preprocessed_data(self, directory):
        if not self.__graph.savePreprocessedData(directory):
            raise ValueError("Saving data to disk failed.")
        return self



from accelera.src.utils.py2cpp_converter import py2cpp_converter


class TestPy2CppConverter:
    def test_converts_print(self):
        cpp = py2cpp_converter('print("hi")')
        assert "#include <iostream>" in cpp
        assert "int main() {" in cpp
        assert "std::cout" in cpp
        assert '"hi"' in cpp
        assert "std::endl" in cpp

    def test_converts_assignment_and_augassign(self):
        cpp = py2cpp_converter("x = 1\nx += 2")
        assert "int main() {" in cpp
        assert "int x = 1;" in cpp
        assert "x += 2;" in cpp

    def test_converts_for_range(self):
        cpp = py2cpp_converter("for i in range(0, 3):\n    print(i)")
        assert "int main() {" in cpp
        assert "for (int i = 0; i < 3; i += 1)" in cpp
        assert "std::cout" in cpp

    def test_converts_function_def(self):
        cpp = py2cpp_converter(
            """
def add(a, b):
    c = a + b
    return c
""".strip()
        )
        assert "auto add(auto a, auto b)" in cpp
        assert "auto c = (a + b);" in cpp
        assert "return c;" in cpp
        assert "int main() {" in cpp
        assert cpp.index("auto add(auto a, auto b)") < cpp.index("int main() {")



import numpy as np
import pytest
from PIL import Image

from accelera.src.automl.core.segmentation_image_testing_preprocessing import (
    SegmentationImageTestingPreprocessing,
)
from accelera.src.utils.preprocessing import save_pickle


class TestSegmentationImageTestingPreprocessing:
    def test_constructor(self, tmp_path):
        data_info = {"image_size": (125, 125), "binary_mask_threshold": 128}
        save_pickle(tmp_path, data_info, "data_info.pkl")

        with pytest.raises(
            ValueError, match="Image paths must be list of paths not none"
        ):
            SegmentationImageTestingPreprocessing(
                image_paths=None, folder_path=tmp_path
            )

        with pytest.raises(ValueError, match="Image paths must be list of paths"):
            SegmentationImageTestingPreprocessing(
                image_paths="path", folder_path=tmp_path
            )
        with pytest.raises(ValueError, match="Image paths is empty list"):
            SegmentationImageTestingPreprocessing(
                image_paths=[], folder_path=tmp_path
            )

        with pytest.raises(ValueError, match="masks must be list of masks paths"):
            SegmentationImageTestingPreprocessing(
                image_paths=["path"], image_masks=0, folder_path=tmp_path
            )

        with pytest.raises(
            ValueError, match="image paths length must equal masks length"
        ):
            SegmentationImageTestingPreprocessing(
                image_paths=["path1", "path2"],
                image_masks=[0],
                folder_path=tmp_path,
            )
        with pytest.raises(ValueError, match="There is no valid image exists"):
            SegmentationImageTestingPreprocessing(
                image_paths=["path1", "path2"],
                image_masks=["mask", "mask"],
                folder_path=tmp_path,
            )
        images = []
        masks = []
        for i in range(5):
            (tmp_path / f"invalid_{i}.png").touch()
            images.append(str(tmp_path / f"invalid_{i}.png"))
            masks.append(str(tmp_path / f"invalid_{i}.png"))
            img = Image.fromarray(
                np.random.randint(0, 256, (225, 225, 3), dtype=np.uint8)
            )
            img.save(tmp_path / f"img_{i}.png")
            images.append(str(tmp_path / f"img_{i}.png"))
            masks.append(str(tmp_path / f"img_{i}.png"))
        preprocesseor = SegmentationImageTestingPreprocessing(
            image_paths=images,
            image_masks=masks,
            folder_path=tmp_path,
        )
        assert len(preprocesseor.valid_images) == 5
        assert len(preprocesseor.invalid_images) == 5

    def test_common_preprocessing(self, tmp_path):
        data_info = {"image_size": (125, 125), "binary_mask_threshold": 128}
        save_pickle(tmp_path, data_info, "data_info.pkl")

        images = []
        masks = []
        for i in range(5):
            img = Image.fromarray(
                np.random.randint(0, 256, (225, 225, 3), dtype=np.uint8)
            )
            img.save(tmp_path / f"img_{i}.png")
            images.append(str(tmp_path / f"img_{i}.png"))
            masks.append(str(tmp_path / f"img_{i}.png"))

        testing_loader, invalid_paths = SegmentationImageTestingPreprocessing(
            image_paths=images,
            image_masks=None,
            folder_path=tmp_path,
        ).common_preprocessing()
        assert testing_loader is not None
        assert len(invalid_paths) == 0
        batch = next(iter(testing_loader))
        assert len(batch) == 4
        testing_loader, invalid_paths = SegmentationImageTestingPreprocessing(
            image_paths=images,
            image_masks=masks,
            folder_path=tmp_path,
        ).common_preprocessing()
        assert testing_loader is not None
        assert len(invalid_paths) == 0
        batch = next(iter(testing_loader))
        assert len(batch) == 2
        assert len(batch[0]) == 4
        assert len(batch[1]) == 4



from sklearn.base import BaseEstimator
from sklearn.base import ClassifierMixin


class CustomClassifier(BaseEstimator, ClassifierMixin):
    def fit(self, X, y):
        raise NotImplementedError("CustomClassifier.fit is not implemented yet.")

    def predict(self, X):
        raise NotImplementedError("CustomClassifier.predict is not implemented yet.")



import os

import matplotlib.pyplot as plt
import numpy as np

from accelera.src.automl.wrappers.graph_base import GraphBase


class ClassificationImagesAfterLoader(GraphBase):
    def __init__(
        self,
        images,
        labels,
        label2class_mapping,
        folder_path,
        title="",
        file_name="",
    ):
        super().__init__(folder_path)
        self.images = images
        self.labels = labels
        self.title = title
        self.file_name = file_name
        self.label2class_mapping = label2class_mapping

    def build_graph(self):
        cols = len(self.images)
        fig, ax = plt.subplots(1, cols, figsize=(4 * cols, 4))
        if cols == 1:
            ax = [ax]
        fig.suptitle(self.title, fontsize=20)
        for i, img in enumerate(self.images):
            img = img.permute(1, 2, 0).numpy()
            img = np.clip(img, 0, 1)
            label = self.labels[i].item()
            class_name = self.label2class_mapping[label]
            ax[i].imshow(img)
            ax[i].set_title(class_name)
            ax[i].axis("off")

        plt.tight_layout()
        plt.savefig(os.path.join(self.folder_path, f"{self.file_name}.png"))
        plt.close()



import os
import re
import textwrap
import xml.etree.ElementTree as ET

import pandas as pd
from graphviz import Digraph

from accelera.src.accelera_pipe.wrappers.graph_pipeline_report import (
    GraphPipelineReport,
)


class GraphReport(GraphPipelineReport):
    def __init__(self, folderpath, xmlpath, results):
        super().__init__(folderpath, results)
        self.xmlpath = xmlpath
        self.metric_ids = []

    def get_branches(self, nodes, tree, current_node_id="0", path=None):
        if path is None:
            path = []
        if nodes[current_node_id]["node_type"] == "METRIC":
            return [{"path": path, "metrics": nodes[current_node_id]}]

        path.append(nodes[current_node_id])
        if current_node_id not in tree:
            return [{"path": path, "metrics": {}}]

        all_branches = []
        for child_id in tree[current_node_id]:
            child_branches = self.get_branches(nodes, tree, child_id, list(path))
            all_branches.extend(child_branches)
        return all_branches

    def grouby_branches(self, branches):
        grouped_branches = {}
        for branch in branches:
            key = tuple(node["node_id"] for node in branch["path"])
            if key not in grouped_branches:
                grouped_branches[key] = branch["path"]
            if branch["metrics"]:
                grouped_branches[key].append(branch["metrics"])
        return list(grouped_branches.values())

    def display_branch(self, branch, title):
        ids = [node["node_id"] for node in branch]
        names = [node["node_name"] for node in branch]
        types = [node["node_type"] for node in branch]
        data = {"Node ID": ids, "Node Name": names, "Node Type": types}
        table = pd.DataFrame(data).to_html(index=False, border=1, justify="center")
        content = f"<div>\n<h3> {title}</h3>\n{table}\n</div>\n"
        return content

    def display_branches(self, branches, best_branch):
        branches = self.grouby_branches(branches)
        content = (
            "<div>\n<h2> Pipeline Branches</h2>\n<div class='metric-container'>\n"
        )
        branch_id = 1
        for branch in branches:
            title = f"Branch {branch_id}"
            content += self.display_branch(branch, title)
            branch_id += 1
        content = content + "</div>\n"
        if best_branch:
            content += self.display_branch(best_branch, "Best Branch")
        return content + "</div>\n"

    def create_graph_img(self):
        try:
            xml_tree = ET.parse(self.xmlpath)
            root = xml_tree.getroot()
        except FileNotFoundError:
            raise FileNotFoundError(
                f"XML file: {self.xmlpath} not found please run serialize function"
            )
        except ET.ParseError as e:
            raise ValueError(f"Invalid XML file {self.xmlpath}: {e}")

        graph = Digraph()
        graph.attr("graph")
        nodes = {}
        tree = {}
        best_branch = []

        for layer in root.findall("layers/layer"):
            node_id = layer.get("id")
            node_name = layer.get("name")
            node_name = re.sub(r"_\d+", "", node_name)
            node_type = layer.get("type")
            selected_in_path = layer.get("selected_in_path")
            color = "red" if selected_in_path == "true" else "blue"
            if node_type == "METRIC":
                self.metric_ids.append(node_id)
            node_full_name = f"{node_id}\\n{node_type}\\n{node_name}"
            graph.node(node_id, label=node_full_name, color=color)
            node_object = {
                "node_id": node_id,
                "node_type": node_type,
                "node_name": node_name,
            }
            nodes[node_id] = node_object
            if selected_in_path == "true":
                best_branch.append(node_object)

        for edge in root.findall("edges/edge"):
            from_id, to_id = edge.get("from-layer"), edge.get("to-layer")
            graph.edge(from_id, to_id)
            if from_id in tree:
                tree[from_id].append(to_id)
            else:
                tree[from_id] = [to_id]
        img_path = os.path.join(self.folderpath, "graph")
        graph.render(img_path, format="png", cleanup=True)
        return nodes, tree, best_branch

    def execute(self):
        nodes, tree, best_branch = self.create_graph_img()
        branches = self.get_branches(nodes, tree)
        metric_content = self.metric_display()
        content = textwrap.dedent(
            """\
        <p>This is the automated report for the
        pipeline created using <i>Accelera</i>.</p>
        <p>It includes both:</p>
        <ul>
        <li>A graphical representation of the pipeline structure</li>
        <li>A summary of the resulting metrics</li>
        </ul>
        <h2>Graphical Representation</h2>
        <img src='graph.png' alt="Graphical Representation" 
        style="width:100%; max-width:1200px;"/>
        <h3>Description</h3>
        <p>
        The graph illustrates the <b>nodes</b> in the
        pipeline and their <b>connections</b>.</p>
        <ul>
        <li>Each node is labeled with its:
        <ul>
            <li><b>ID</b></li>
            <li><b>Type</b> (one of: Input, Preprocess,
            Model, Predict, Merge, Metric)</li>
            <li><b>Name</b></li>
            </ul>
        </li>
        <li>Node colors:
            <ul>
            <li><b>Red nodes</b> → selected in the final execution path</li>
            <li><b>Blue nodes</b> → present in the
            structure but not part of the final path</li>
            </ul>
        </li>
        </ul>
        """
        )
        branch_content = self.display_branches(branches, best_branch)
        content = (
            self.start_content
            + "\n"
            + content
            + "\n"
            + branch_content
            + "\n"
            + metric_content
            + self.end_content
        )
        self.create_html_file(content)



import os

from torch.utils.data import DataLoader

from accelera.src.automl.core.image_training_preprocessing import (
    ImageTrainingPreprocessing,
)
from accelera.src.automl.core.segmentation_image_dataset import (
    SegmentationImageDataset,
)
from accelera.src.automl.wrappers.display_sample_images_segmentation import (
    DisplaySampleImagesSegmentation,
)
from accelera.src.automl.wrappers.image_preprocessing_report import (
    ImagePreprocessingReport,
)
from accelera.src.automl.wrappers.segmentation_data_summary import (
    Segmentation_data_summary,
)
from accelera.src.automl.wrappers.segmentation_images_after_loader import (
    SegmentationImagesAfterLoader,
)
from accelera.src.utils.preprocessing import check_path_exists
from accelera.src.utils.preprocessing import is_valid_image
from accelera.src.utils.preprocessing import save_pickle


class SegmentationImageTrainingPreprocessing(ImageTrainingPreprocessing):
    def __init__(
        self,
        training_folder_images,
        training_folder_masks,
        folder_path,
        binary_mask_threshold=128,
        validation_folder_images=None,
        validation_folder_masks=None,
        split_training=False,
        val_size=0.2,
        random_state=42,
        images_size=(224, 224),
        augment=False,
        batch_size=16,
        augmentation_probability=0.5,
        horizontal_flip=False,
        vertical_flip=False,
        rotation=False,
        rotation_angle=30,
        brightness=False,
        brightness_factors=(0.8, 1.2),
        contrast=False,
        contrast_factors=(0.8, 1.2),
    ):
        super().__init__(
            training_folder_images,
            folder_path,
            validation_folder_images,
            split_training,
            val_size,
            random_state,
            images_size,
            augment,
            batch_size,
            augmentation_probability,
            horizontal_flip,
            vertical_flip,
            rotation,
            rotation_angle,
            brightness,
            brightness_factors,
            contrast,
            contrast_factors,
        )

        self.training_folder_masks = training_folder_masks
        self.validation_folder_masks = validation_folder_masks
        self.binary_mask_threshold = binary_mask_threshold
        (
            self.validation_folder_images_paths,
            self.validation_folder_masks_paths,
        ) = (
            None,
            None,
        )
        self.training_invalid_images_paths = []
        self.training_invalid_masks_paths = []
        self.validation_invalid_images_paths = []
        self.validation_invalid_masks_paths = []
        if self.training_folder_masks is None:
            raise ValueError("Training folder masks must be not null")
        check_path_exists(self.training_folder_masks, "")
        if self.training_folder == self.training_folder_masks:
            raise ValueError(
                "training folder images and training folder masks must be different"
            )

        if self.binary_mask_threshold is None:
            raise ValueError(
                "you must add binary_mask_threshold value if pixel value >= "
                "binary_mask_threshold it will be 1 and 0 else"
            )
        if not (
            isinstance(self.binary_mask_threshold, int)
            and (0 <= self.binary_mask_threshold <= 255)
        ):
            raise ValueError(
                "binary_mask_threshold must be integer between 0 and 255"
            )

        if self.validation_folder is not None:
            if self.validation_folder_masks is None:
                raise ValueError("Validation folder masks must be not null")
            check_path_exists(self.validation_folder_masks, "")
            if self.validation_folder == self.validation_folder_masks:
                raise ValueError(
                    "validation folder images and validation folder masks "
                    "must be different"
                )
        data_info = {
            "image_size": self.image_size,
            "binary_mask_threshold": self.binary_mask_threshold,
        }
        save_pickle(self.folder_path, data_info, "data_info.pkl")

    def data_preparing(
        self,
        images_folder_path,
        masks_folder_path,
        invalid_images_paths,
        invalid_masks_paths,
    ):
        images_paths = []
        masks_paths = []
        images_dict = {
            os.path.splitext(image_path)[0]: image_path
            for image_path in os.listdir(images_folder_path)
        }
        masks_dict = {
            os.path.splitext(mask_path)[0]: mask_path
            for mask_path in os.listdir(masks_folder_path)
        }
        matches = set(images_dict.keys()) & set(masks_dict.keys())
        if len(matches) == 0:
            raise ValueError("no matches between masks and images names")

        for key in sorted(matches):
            image_path = os.path.join(images_folder_path, images_dict[key])
            mask_path = os.path.join(masks_folder_path, masks_dict[key])
            if is_valid_image(image_path) and is_valid_image(mask_path):
                images_paths.append(image_path)
                masks_paths.append(mask_path)
            else:
                invalid_images_paths.append(image_path)
                invalid_masks_paths.append(mask_path)
        if len(images_paths) == 0:
            raise ValueError("There is no valid path")
        return images_paths, masks_paths

    def data_overview(self):
        train_df = self.get_sample_random(
            "training_folder",
            self.training_folder_images_paths,
            self.training_folder_masks_paths,
        )
        self.report_data["data_overview"] = {}
        self.report_data["data_overview"]["training_folder"] = {
            "images_len": len(self.training_folder_images_paths),
            "random_sample": train_df.head(),
            "invalid_len": len(self.training_invalid_images_paths),
            "invalid_images": self.training_invalid_images_paths,
            "invalid_masks": self.training_invalid_masks_paths,
        }

        if self.validation_folder is not None:
            val_df = self.get_sample_random(
                "validation_folder",
                self.validation_folder_images_paths,
                self.validation_folder_masks_paths,
            )
            self.report_data["data_overview"]["validation_folder"] = {
                "images_len": len(self.validation_folder_images_paths),
                "random_sample": val_df.head(),
                "invalid_len": len(self.validation_invalid_images_paths),
                "invalid_images": self.validation_invalid_images_paths,
                "invalid_masks": self.validation_invalid_masks_paths,
            }

    def make_graphs_data_summary(self):
        Segmentation_data_summary(
            self.training_paths,
            self.training_invalid_images_paths,
            folder_path=self.folder_path,
            title="Training Folder Summary",
            file_name="training_folder_summary",
        ).build_graph()
        self.report_data["graphs"]["images_name"].append("training_folder_summary")
        if self.validation_folder is not None:
            Segmentation_data_summary(
                self.validation_paths,
                self.validation_invalid_images_paths,
                folder_path=self.folder_path,
                title="Validation Folder Summary",
                file_name="validation_folder_summary",
            ).build_graph()
        self.report_data["graphs"]["images_name"].append("validation_folder_summary")

    def make_garphs_sample(self):
        DisplaySampleImagesSegmentation(
            self.training_folder_images_paths,
            self.training_folder_masks_paths,
            self.folder_path,
            title="Random Samples of Training Folder",
            file_name="training_folder_random_samples",
        ).build_graph()
        self.report_data["graphs"]["images_name"].append(
            "training_folder_random_samples"
        )
        if self.validation_folder is not None:
            DisplaySampleImagesSegmentation(
                self.validation_folder_images_paths,
                self.validation_folder_masks_paths,
                self.folder_path,
                title="Random Samples of Validation Folder",
                file_name="validation_folder_random_samples",
            ).build_graph()
            self.report_data["graphs"]["images_name"].append(
                "validation_folder_random_samples"
            )
        if self.split_training:
            DisplaySampleImagesSegmentation(
                self.training_paths,
                self.training_labels,
                self.folder_path,
                title="Samples of Training Data After Splitting",
                file_name="training_after_splitting_random_samples",
            ).build_graph()
            self.report_data["graphs"]["images_name"].append(
                "training_after_splitting_random_samples"
            )
            DisplaySampleImagesSegmentation(
                self.validation_paths,
                self.validation_labels,
                self.folder_path,
                title="Samples of Validation Data After Splitting",
                file_name="validation_after_splitting_random_samples",
            ).build_graph()
            self.report_data["graphs"]["images_name"].append(
                "validation_after_splitting_random_samples"
            )

    def make_graphs_loader(self):
        training_images, training_labels = next(iter(self.training_loader))
        n_samples = min(5, len(training_images))

        training_images, training_labels = (
            training_images[:n_samples],
            training_labels[:n_samples],
        )
        SegmentationImagesAfterLoader(
            training_images,
            training_labels,
            self.folder_path,
            title="Samples of Training Data After Data Loader",
            file_name="training_after_data_loader_samples",
        ).build_graph()
        self.report_data["graphs"]["images_name"].append(
            "training_after_data_loader_samples"
        )
        if self.validation_loader is not None:
            validation_images, validation_labels = next(iter(self.validation_loader))
            n_samples = min(5, len(validation_images))
            validation_images, validation_labels = (
                validation_images[:n_samples],
                validation_labels[:n_samples],
            )
            SegmentationImagesAfterLoader(
                validation_images,
                validation_labels,
                self.folder_path,
                title="Samples of Validation Data After Data Loader",
                file_name="validation_after_data_loader_samples",
            ).build_graph()
            self.report_data["graphs"]["images_name"].append(
                "validation_after_data_loader_samples"
            )

    def make_graphs(self):
        self.report_data["graphs"] = {
            "folder_path": self.folder_path,
            "images_name": [],
        }
        self.make_graphs_data_summary()
        self.make_garphs_sample()
        self.make_graphs_loader()

    def get_loaders(self):
        training_dataset = SegmentationImageDataset(
            self.training_paths,
            self.training_labels,
            self.image_size,
            self.binary_mask_threshold,
            self.augment,
            self.augmentation_probability,
            self.horizontal_flip,
            self.vertical_flip,
            self.rotation,
            self.rotation_angle,
            self.brightness,
            self.brightness_factors,
            self.contrast,
            self.contrast_factors,
        )
        self.training_loader = DataLoader(
            training_dataset, batch_size=self.batch_size, shuffle=True
        )

        self.validation_loader = None
        if self.validation_paths is not None:
            validation_dataset = SegmentationImageDataset(
                self.validation_paths,
                self.validation_labels,
                self.image_size,
                self.binary_mask_threshold,
                False,
                self.augmentation_probability,
                self.horizontal_flip,
                self.vertical_flip,
                self.rotation,
                self.rotation_angle,
                self.brightness,
                self.brightness_factors,
                self.contrast,
                self.contrast_factors,
            )
            self.validation_loader = DataLoader(
                validation_dataset, batch_size=self.batch_size, shuffle=False
            )

    def common_preprocessing(self):
        (
            self.training_folder_images_paths,
            self.training_folder_masks_paths,
        ) = self.data_preparing(
            self.training_folder,
            self.training_folder_masks,
            self.training_invalid_images_paths,
            self.training_invalid_masks_paths,
        )
        if self.validation_folder is not None:
            (
                self.validation_folder_images_paths,
                self.validation_folder_masks_paths,
            ) = self.data_preparing(
                self.validation_folder,
                self.validation_folder_masks,
                self.validation_invalid_images_paths,
                self.validation_invalid_masks_paths,
            )

        self.data_overview()
        self.splitting(
            self.training_folder_images_paths,
            self.validation_folder_images_paths,
            self.training_folder_masks_paths,
            self.validation_folder_masks_paths,
        )
        self.get_loaders()
        self.make_graphs()

        report = ImagePreprocessingReport(self.folder_path, self.report_data)
        report.execute()
        return self.training_loader, self.validation_loader



import pandas as pd

from accelera.src.accelera_pipe.core.metric_display import MetricDisplay


class DisplayDict(MetricDisplay):
    def __init__(self, metric_name, values):
        super().__init__(metric_name, values)

    def execute(self):
        content = (
            "<div>\n"
            f"<h3>Metric name: {self.metric_name}</h3>\n"
            "<div class='metric-container'>\n"
        )
        for value in self.values:
            table = (
                pd.DataFrame(value["result"])
                .transpose()
                .round(3)
                .to_html(border=1, justify="center")
            )
            new_content = "<div>\n"
            new_content += "<h4>\n"
            new_content += (
                f"Metric id :{value['metric id']}</h4>\n {table}\n</div>\n"
            )
            content = content + new_content
        content = content + "</div>\n</div>\n"

        return content



from abc import ABC
from abc import abstractmethod


class MetricDisplay(ABC):
    def __init__(self, metric_name, values):
        self.metric_name = metric_name
        self.values = values

    def handle_name(self, field_name, required_lenght):
        if self.values[0][field_name]:
            if not isinstance(self.values[0][field_name], list):
                raise TabError(f"'{field_name} must be a list'")
            if len(self.values[0][field_name]) != required_lenght:
                raise ValueError(
                    f"length of {field_name} dose not match the "
                    "length of the result from the metric "
                    f"The length of {field_name} "
                    f"of the metric is {required_lenght} "
                    f"but the length of the '{field_name}' "
                    f"is {len(self.values[0][field_name])}'"
                )
            names = self.values[0][field_name]
            return names
        return list(range(required_lenght))

    @abstractmethod
    def execute(self):
        pass



from accelera.src.accelera_pipe.core.executed_graph import ExecutedGraph
from accelera.src.accelera_pipe.core.node import Node
from accelera.src.accelera_pipe.core.pipeline_base import PipelineBase
from accelera.src.utils.accelera_utils import execute_fit
from accelera.src.utils.accelera_utils import get_correct_metric_class
from accelera.src.utils.accelera_utils import get_metric_object


class Pipeline(PipelineBase):
    def __init__(self):
        super().__init__(_graph=None)

    def __call__(
        self, X, y=None, select_strategy: str = "all", custom_strategy=None
    ):
        results = self._PipelineBase__graph.execute(
            X,
            y=y,
            select_strategy=select_strategy,
            custom_strategy=custom_strategy,
        )
        executed_graph = ExecutedGraph(results[0])
        predictions = results[1:]
        return predictions, executed_graph

    def preprocess(self, name, func, branch=False):
        func_params = {"func": func, "execute_fit": execute_fit}
        if branch:
            return Node("preprocess", name, func_params)

        self._PipelineBase__graph.add_node(
            self.types["preprocess"], name, func_params
        )
        return self

    def model(self, name, model, branch=False):
        model_params = {
            "model": model,
            "execute_fit": execute_fit,
        }
        if branch:
            return Node("model", name, model_params)

        self._PipelineBase__graph.add_node(self.types["model"], name, model_params)
        return self

    def predict(
        self,
        name,
        test_data,
        output_func="predict",
        positive_class=-1,
        branch=False,
    ):
        predict_params = {
            "test_data": test_data,
            "output_func": output_func,
            "positive_class": positive_class,
        }
        if branch:
            return Node("predict", name, predict_params)

        self._PipelineBase__graph.add_node(
            self.types["predict"], name, predict_params
        )
        return self

    def metric(
        self,
        name,
        metric_name,
        y_true=None,
        plot_func=None,
        branch=False,
        labels_name=None,
        headers_name=None,
        **params,
    ):
        metric_func = get_metric_object(metric_name)

        if metric_func is not None:
            metric_obj = get_correct_metric_class(
                name,
                metric_func,
                y_true,
                plot_func,
                labels_name,
                headers_name,
                **params,
            )
            if metric_obj is None:
                raise ValueError(
                    f"Metric '{metric_name}' is incompatible with "
                    "the supervised or unsupervised metric structure."
                )

            if branch:
                return Node("metric", name, metric_obj)

            self._PipelineBase__graph.add_node(
                self.types["metric"], name, metric_obj
            )
            return self
        else:
            raise ValueError(f"Metric '{metric_name}' is not recognized.")

    def merge(self, name, strategy="hard_voting", branch=False):
        if branch:
            return Node("merge", name, strategy)

        self._PipelineBase__graph.add_node(self.types["merge"], name, strategy)
        return self

    def branch(self, name, *branches):
        branches_to_send = []
        node_types = []
        node_names = []

        for branch in branches:
            branch_objects = []

            if isinstance(branch, (list, tuple)):
                branch_iter = branch
            elif hasattr(branch, "__iter__") and not isinstance(branch, str):
                branch_iter = branch
            else:
                branch_iter = [branch]

            for node in branch_iter:
                if isinstance(node, Node):
                    branch_objects.append(node.obj)
                    if node.node_type == "merge":
                        raise ValueError(
                            "Branching with Merge nodes is not "
                            "implemented yet. Please add merge nodes "
                            "outside of branches."
                        )
                    node_types.append(node.node_type.upper())
                    node_names.append(node.name)
                else:
                    raise ValueError(
                        "All arguments to branch() must be"
                        " Node instances. Use the 'branch=True' "
                        "argument when adding nodes. "
                        f"Got: {type(node).__name__}"
                    )

            branches_to_send.append(branch_objects)

        self._PipelineBase__graph.split(
            name, branches_to_send, node_types, node_names
        )
        return self



import os

import matplotlib.pyplot as plt
import pandas as pd

from accelera.src.accelera_pipe.core.metric_display import MetricDisplay

plt.style.use("dark_background")


class DisplaySingleNumber(MetricDisplay):
    def __init__(self, metric_name, values, folderpath):
        super().__init__(metric_name, values)
        self.folderpath = folderpath

    def execute(self):
        ids, results = (
            [value["metric id"] for value in self.values],
            [value["result"] for value in self.values],
        )
        data = {
            "Metric ID": ids,
            "Metric Value": results,
        }
        img_path = os.path.join(self.folderpath, f"{self.metric_name}.png")
        table = (
            pd.DataFrame(data)
            .transpose()
            .round(3)
            .to_html(border=1, justify="center")
        )
        plt.plot(ids, results, marker="o")
        plt.xlabel("Metric ID")
        plt.ylabel("Metric Value")
        plt.title(f"{self.metric_name}")
        plt.savefig(img_path)
        plt.close()
        content = (
            "<div>\n"
            f"<h3>Metric name: {self.metric_name}</h3>\n"
            f"<div><p>Table</p> \n{table}\n</div>\n"
            f"<div><p>Graph</p> \n"
            f"<img src='{self.metric_name}.png' "
            f"alt='{self.metric_name}' /></div>\n"
            "</div>\n"
        )
        return content



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from accelera.src.utils import accelera_utils as accelera_utils
from accelera.src.utils import accelera_utils_test as accelera_utils_test
from accelera.src.utils import array_utils as array_utils
from accelera.src.utils import array_utils_test as array_utils_test
from accelera.src.utils import dataset_retriever as dataset_retriever
from accelera.src.utils import dataset_retriever_test as dataset_retriever_test
from accelera.src.utils import parallelizer as parallelizer
from accelera.src.utils import parallelizer_test as parallelizer_test
from accelera.src.utils import preprocessing as preprocessing
from accelera.src.utils import py2cpp_converter as py2cpp_converter
from accelera.src.utils import py2cpp_converter_test as py2cpp_converter_test



import pandas as pd

from accelera.src.automl.core.preprocessing_base import PreprocessingBase


class TabularPreprocessingBase(PreprocessingBase):
    def __init__(self, df, folder_path=None):
        self.df = df
        if df is None:
            raise ValueError("Dataframe cannot be None")
        if not isinstance(df, pd.DataFrame):
            raise ValueError("df must be a pandas DataFrame")
        super().__init__(folder_path)



import shutil
import tempfile

import numpy as np
import pandas as pd
import pytest
from sklearn.preprocessing import LabelEncoder

from accelera.src.automl.core.text_testing_preprocesing import (
    TextTestingPreprocessing,
)
from accelera.src.utils.preprocessing import save_pickle


class TestTextTestingPreprocessing:
    @pytest.fixture(autouse=True)
    def initializetion(self):
        self.temp_dir = tempfile.mkdtemp()
        self.data_info = {
            "target_col": "class",
            "text_col": "text",
            "target_mode": "pos",
        }
        self.train = pd.DataFrame(
            {
                "text": [
                    "Hello ",
                    "It is not great",
                    "great like hello",
                    "like great hello",
                    "i dont like it",
                ],
                "class": ["pos", "neg", "pos", "pos", "neg"],
            }
        )
        self.test = pd.DataFrame(
            {
                "text": ["welcome ", "It is not bad", "i  like it"],
                "class": ["pos", None, "pos"],
            }
        )
        self.target_preprocessor = LabelEncoder()
        self.target_preprocessor.fit_transform(self.train["class"])

        yield self.temp_dir
        shutil.rmtree(self.temp_dir)

    def test_text_errors_test(self):
        with pytest.raises(ValueError):
            TextTestingPreprocessing(df=None, folder_path=self.temp_dir)

        with pytest.raises(ValueError):
            TextTestingPreprocessing(df=self.test, folder_path=None)

        with pytest.raises(ValueError):
            TextTestingPreprocessing(df=self.test, folder_path=self.temp_dir)
        save_pickle(
            self.temp_dir, self.target_preprocessor, "target_preprocessor.pkl"
        )
        with pytest.raises(ValueError):
            TextTestingPreprocessing(df=self.test, folder_path=self.temp_dir)
        save_pickle(self.temp_dir, {}, "training_preprocessor.pkl")
        with pytest.raises(ValueError):
            TextTestingPreprocessing(df=self.test, folder_path=self.temp_dir)
        self.data_info["text_col"] = "review"
        save_pickle(self.temp_dir, self.data_info, "data_info.pkl")
        with pytest.raises(ValueError):
            TextTestingPreprocessing(df=self.test, folder_path=self.temp_dir)

        self.data_info["target_col"] = "class"
        self.data_info["text_col"] = "class"
        save_pickle(self.temp_dir, self.data_info, "data_info.pkl")
        with pytest.raises(ValueError):
            TextTestingPreprocessing(df=self.test, folder_path=self.temp_dir)
        with pytest.raises(ValueError):
            tp = TextTestingPreprocessing(df=self.test, folder_path=self.temp_dir)
            tp.feature_preprocessing()
        save_pickle(self.temp_dir, {}, "target_preprocessor.pkl")
        with pytest.raises(ValueError):
            tp = TextTestingPreprocessing(df=self.test, folder_path=self.temp_dir)
            tp.target_preprocessing()

    def test_text_target_preprocessing_test(self):
        self.data_info["target_col"] = "review"
        self.data_info["text_col"] = "text"
        y = self.test["class"]
        save_pickle(self.temp_dir, self.data_info, "data_info.pkl")
        save_pickle(self.temp_dir, {}, "training_preprocessor.pkl")
        save_pickle(
            self.temp_dir, self.target_preprocessor, "target_preprocessor.pkl"
        )

        tp = TextTestingPreprocessing(self.test, self.temp_dir)
        y_test = tp.target_preprocessing()
        assert tp.features_only
        assert y_test is None

        self.data_info["target_col"] = "class"
        self.data_info["text_col"] = "text"
        self.data_info["target_mode"] = "pos"
        save_pickle(self.temp_dir, self.data_info, "data_info.pkl")
        tp = TextTestingPreprocessing(self.test, self.temp_dir)
        y_test = tp.target_preprocessing()
        assert not tp.features_only
        assert y_test is not None
        y = y.fillna("pos")
        y = self.target_preprocessor.transform(y)
        assert np.allclose(y, y_test)
        assert tp.data_info["target_mode"] == "pos"



import numpy as np
import pandas as pd

from accelera.src.accelera_pipe.core.metric_display import MetricDisplay


class DisplayMultiArray(MetricDisplay):
    def __init__(self, metric_name, values):
        super().__init__(metric_name, values)

    def execute(self):
        content = f"<div>\n<h3>Metric name: {self.metric_name}</h3>\n"
        labels_name = self.handle_name(
            "labels_name", self.values[0]["result"].shape[0]
        )
        headers_name = self.handle_name(
            "headers_name", self.values[0]["result"].shape[1]
        )
        for value in self.values:
            data = {"labels": labels_name}
            for row_idx in range(value["result"].shape[0]):
                for col_idx in range(len(value["result"][row_idx])):
                    array_str = np.array2string(
                        np.array(value["result"][row_idx][col_idx]),
                        separator=", ",
                        max_line_width=80,
                    )
                    if headers_name[col_idx] not in data:
                        data[headers_name[col_idx]] = [array_str]
                    else:
                        data[headers_name[col_idx]].append(array_str)

            table = pd.DataFrame(data).to_html(
                index=False, border=1, justify="center"
            )
            new_content = (
                "<div>\n"
                "<h4>\n"
                f"Metric id :{value['metric id']}</h4>\n\n"
                f"{table}\n"
                "</div>\n"
            )
            content = content + new_content + "</div>\n"
        content = content
        return content



import numpy as np
import pytest

from accelera.src.utils.array_utils import convert_to_array
from accelera.src.utils.array_utils import get_array_info
from accelera.src.utils.array_utils import validate_array_shape


class TestConvertToArray:
    def test_convert_none_returns_none(self):
        result = convert_to_array(None)
        assert result is None

    def test_convert_list_to_array(self):
        input_list = [1, 2, 3, 4]
        result = convert_to_array(input_list)

        assert isinstance(result, np.ndarray)
        np.testing.assert_array_equal(result, np.asarray(input_list))

    def test_convert_with_dtype(self):
        input_list = [1.5, 2.7, 3.9]
        result = convert_to_array(input_list, dtype=np.int32)

        assert result.dtype == np.int32
        np.testing.assert_array_equal(result, np.asarray(input_list, dtype=np.int32))


class TestValidateArrayShape:
    def test_validate_matching_shapes(self):
        array = np.array([[1, 2], [3, 4], [5, 6]])
        validate_array_shape(array, 3)

    def test_validate_mismatched_shapes_raises_error(self):
        array = np.array([[1, 2], [3, 4]])

        with pytest.raises(ValueError, match="must have the same number of samples"):
            validate_array_shape(array, 5)

    def test_validate_with_custom_names(self):
        array = np.array([1, 2, 3])
        names = ["predictions", "targets"]

        with pytest.raises(ValueError) as exc_info:
            validate_array_shape(array, 5, names)

        error_msg = str(exc_info.value)
        assert "predictions" in error_msg
        assert "targets" in error_msg


class TestGetArrayInfo:
    def test_get_info_none_input(self):
        """Test getting info for None input."""
        result = get_array_info(None)
        expected = {"shape": None, "dtype": None, "ndim": None, "size": None}
        assert result == expected

    def test_get_info_array(self):
        array = np.array([[1, 2, 3], [4, 5, 6]])
        result = get_array_info(array)

        assert result["shape"] == (2, 3)
        assert result["ndim"] == 2
        assert result["size"] == 6

    def test_get_info_list_input(self):
        result = get_array_info([1, 2, 3])

        assert result["shape"] == (3,)
        assert result["ndim"] == 1
        assert result["size"] == 3



import json
import os
import tempfile
from pathlib import Path
from types import SimpleNamespace

import numpy as np
import pytest

from accelera.src.utils.parallelizer import Parallelizer
from accelera.src.utils.parallelizer import extract_loops
from accelera.src.utils.parallelizer import write_loops_to_json


class TestParallelizer:
    def test_classify_returns_prediction(self, monkeypatch):
        parallelizer = Parallelizer()

        class DummyResponse:
            status_code = 200

            def json(self):
                return {"result": "parallel_for"}

        monkeypatch.setattr(
            "accelera.src.utils.parallelizer.requests.post",
            lambda url, json: DummyResponse(),
        )

        result = parallelizer._classify(np.array([1.0, 2.0], dtype=np.float32))

        assert result == "parallel_for"

    def test_generate_omp_pragma_with_loop_none_returns_loop(self):
        parallelizer = Parallelizer()
        loop_code = "for (int i = 0; i < n; ++i) {\n    sum += a[i];\n}"

        result = parallelizer._generate_omp_pragma_with_loop(loop_code, "none")

        assert result == loop_code

    def test_generate_omp_pragma_with_loop_adds_validated_pragma(self, monkeypatch):
        parallelizer = Parallelizer()

        class DummyResponse:
            status_code = 200

            def json(self):
                return {"pragma": "omp parallel for"}

        monkeypatch.setattr(
            "accelera.src.utils.parallelizer.requests.post",
            lambda url, json, timeout: DummyResponse(),
        )
        monkeypatch.setattr(
            "accelera.src.utils.parallelizer.validate_pragma",
            lambda pragma: f"#pragma {pragma}",
        )

        result = parallelizer._generate_omp_pragma_with_loop(
            "for (int i = 0; i < n; ++i) {}", "parallel_for"
        )

        assert result.startswith("#pragma omp parallel for\n")
        assert "for (int i = 0; i < n; ++i) {}" in result

    def test_parallelize_writes_parallelized_output(self, monkeypatch, tmp_path):
        source_file = tmp_path / "sample.c"
        source_file.write_text(
            "int main() {\n"
            "for (int i = 0; i < n; ++i) {\n"
            "    sum += a[i];\n"
            "}\n"
            "return 0;\n"
            "}\n"
        )

        loop_code = "for (int i = 0; i < n; ++i) {\n    sum += a[i];\n}"

        parallelizer = Parallelizer()
        parallelizer.cache_dir = tmp_path / "cache"

        monkeypatch.setattr(
            "accelera.src.utils.parallelizer.extract_loops",
            lambda file_path: [SimpleNamespace()],
        )

        def fake_write_loops_to_json(loops, output_json):
            Path(output_json).write_text(
                json.dumps(
                    [
                        {
                            "code": loop_code,
                            "start_line": 2,
                            "end_line": 4,
                            "type": "for",
                        }
                    ]
                )
            )
            return True

        monkeypatch.setattr(
            "accelera.src.utils.parallelizer.write_loops_to_json",
            fake_write_loops_to_json,
        )
        monkeypatch.setattr(
            "accelera.src.utils.parallelizer.extract_features",
            lambda code: {"dummy": True},
        )
        monkeypatch.setattr(
            "accelera.src.utils.parallelizer.vectorize_features",
            lambda features: np.array([1.0], dtype=np.float32),
        )
        monkeypatch.setattr(parallelizer, "_classify", lambda embedding: "omp")
        monkeypatch.setattr(
            parallelizer,
            "_generate_omp_pragma_with_loop",
            lambda code, cls: f"#pragma omp parallel for\n{code}",
        )

        result = parallelizer.parallelize(str(source_file))

        output_file = tmp_path / "parallelized_sample.c"
        assert result is None
        assert output_file.exists()
        assert "#pragma omp parallel for" in output_file.read_text()


class TestExtractLoops:
    @pytest.fixture
    def simple_cpp_code(self):
        return """
        int main() {
            for (int i = 0; i < 10; i++) {
                // Simple loop
            }
            return 0;
        }
        """

    @pytest.fixture
    def multiple_loops_cpp_code(self):
        return """
        int main() {
            for (int i = 0; i < 10; i++) {}
            
            int j = 0;
            while (j < 5) {
                j++;
            }
            
            return 0;
        }
        """

    def test_extract_loops_simple_file(self, simple_cpp_code):
        loops = extract_loops(simple_cpp_code)

        assert isinstance(loops, list)
        assert len(loops) >= 1
        assert hasattr(loops[0], "type")
        assert hasattr(loops[0], "start_line")
        assert hasattr(loops[0], "end_line")
        assert hasattr(loops[0], "code")

    def test_extract_loops_multiple_loops(self, multiple_loops_cpp_code):
        loops = extract_loops(multiple_loops_cpp_code)

        assert isinstance(loops, list)
        assert len(loops) >= 2

    def test_extract_loops_empty_file(self):
        loops = extract_loops("")
        assert isinstance(loops, list)
        assert len(loops) == 0


class TestWriteLoopsToJson:
    @pytest.fixture
    def real_loops(self):
        content = """
        int main() {
            for (int i = 0; i < 10; i++) {}
            
            int j = 0;
            while (j < 5) {
                j++;
            }
            
            return 0;
        }
        """
        return extract_loops(content)

    def test_write_loops_to_json_success(self, real_loops):
        with tempfile.NamedTemporaryFile(
            mode="w", suffix=".json", delete=False
        ) as f:
            temp_json = f.name

        try:
            result = write_loops_to_json(real_loops, temp_json)
            assert result is True
            assert os.path.exists(temp_json)

            with open(temp_json, "r") as f:
                content = f.read()
                assert "start_line" in content
                assert "end_line" in content
        finally:
            if os.path.exists(temp_json):
                os.remove(temp_json)

    def test_write_loops_to_json_empty_list(self):
        with tempfile.NamedTemporaryFile(
            mode="w", suffix=".json", delete=False
        ) as f:
            temp_json = f.name

        try:
            result = write_loops_to_json([], temp_json)
            assert result is True
            assert os.path.exists(temp_json)
        finally:
            if os.path.exists(temp_json):
                os.remove(temp_json)



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from accelera.src.custom import classifier as classifier
from accelera.src.custom import clusterer as clusterer
from accelera.src.custom import custom_test as custom_test
from accelera.src.custom import regressor as regressor
from accelera.src.custom import transformer as transformer



import numpy as np
import pytest
from sklearn.ensemble import RandomForestClassifier

from accelera.src.accelera_pipe.core.pipeline import Pipeline
from accelera.src.custom.classifier import CustomClassifier

skip_torch = False
try:
    import torch
    from torch import nn
    from torch.utils.data import DataLoader
    from torch.utils.data import TensorDataset

    class TorchDenseModel(CustomClassifier):
        def __init__(self, input_dim=25, output_dim=4, random_state=42):
            torch.manual_seed(random_state)
            self.device = (
                torch.device("cuda")
                if torch.cuda.is_available()
                else pytest.skip("No GPU available, skipping CUDA test.")
            )
            self.model = nn.Sequential(
                nn.Linear(input_dim, 64),
                nn.ReLU(),
                nn.Linear(64, output_dim),
            ).to(self.device)
            self.criterion = nn.CrossEntropyLoss()
            self.optimizer = torch.optim.Adam(self.model.parameters(), lr=0.001)
            self.batch_size = 32
            self.epochs = 10

        def fit(self, X, y):
            X_tensor = torch.tensor(X, dtype=torch.float32).to(self.device)
            y_tensor = torch.tensor(y, dtype=torch.long).to(self.device)
            dataset = TensorDataset(X_tensor, y_tensor)
            dataloader = DataLoader(
                dataset, batch_size=self.batch_size, shuffle=True
            )

            self.model.train()
            for epoch in range(self.epochs):
                for batch_X, batch_y in dataloader:
                    self.optimizer.zero_grad()
                    outputs = self.model(batch_X)
                    loss = self.criterion(outputs, batch_y)
                    loss.backward()
                    self.optimizer.step()

        def predict(self, X):
            self.model.eval()
            X_tensor = torch.tensor(X, dtype=torch.float32).to(self.device)
            with torch.no_grad():
                outputs = self.model(X_tensor)
                _, predicted = torch.max(outputs, 1)
            return predicted.cpu().numpy()
except ImportError:

    class TorchDenseModel:
        pass

    skip_torch = True


class CustomModel(CustomClassifier):
    def __init__(self, random_state=42):
        self.random_state = random_state
        self.centroids_ = None
        self.classes_ = None

    def fit(self, X, y):
        np.random.seed(self.random_state)
        self.classes_ = np.unique(y)

        self.centroids_ = []
        for class_label in self.classes_:
            class_points = X[y == class_label]
            centroid = np.mean(class_points, axis=0)
            self.centroids_.append(centroid)

        self.centroids_ = np.array(self.centroids_)
        return self

    def predict(self, X):
        predictions = []
        for sample in X:
            distances = []
            for centroid in self.centroids_:
                distance = np.sqrt(np.sum((sample - centroid) ** 2))
                distances.append(distance)

            predicted_class = self.classes_[np.argmin(distances)]
            predictions.append(predicted_class)

        return np.array(predictions)


class TestCustomModelIntegration:
    def setup_method(self):
        """Setup test data"""
        self.X = np.random.randn(100, 5)
        self.y = np.random.randint(0, 3, 100)
        self.test_data = np.random.randn(50, 5)

    def test_custom_model_integration(self):
        p = Pipeline()
        p.preprocess("scale", lambda x: x / 10.0)
        p.model("custom_rf", CustomModel(random_state=42))
        p.predict("pred", self.test_data)

        pipeline_result, _ = p(self.X, self.y)

        X_scaled = self.X / 10.0
        test_scaled = self.test_data / 10.0
        manual_model = CustomModel(random_state=42)
        manual_model.fit(X_scaled, self.y)
        manual_result = manual_model.predict(test_scaled)

        pipeline_pred = (
            pipeline_result[0]
            if isinstance(pipeline_result, list)
            else pipeline_result
        )

        assert pipeline_pred.shape == manual_result.shape, (
            f"Shape mismatch: pipeline {pipeline_pred.shape} "
            f"vs manual {manual_result.shape}"
        )

        assert np.array_equal(pipeline_pred, manual_result), (
            f"Predictions don't match.\n"
            f"Pipeline: {pipeline_pred[:10]}\n"
            f"Manual:   {manual_result[:10]}"
        )

    @pytest.mark.skipif(skip_torch, reason="PyTorch is not installed.")
    def test_pipeline_with_cuda_correctness(self):
        p = Pipeline()
        p.preprocess("scale", lambda x: x / 10.0)
        p.branch(
            "models",
            p.model(
                "torch_model",
                TorchDenseModel(input_dim=5, output_dim=4),
                branch=True,
            ),
            p.model(
                "rf",
                RandomForestClassifier(
                    n_estimators=50, random_state=42, max_depth=10
                ),
                branch=True,
            ),
        )
        p.predict("predict", self.test_data, output_func="predict")
        pipeline_result, _ = p(self.X, self.y)

        def p1(x):
            return x / 10.0

        m1 = TorchDenseModel(input_dim=5, output_dim=4)
        m2 = RandomForestClassifier(n_estimators=50, random_state=42, max_depth=10)
        x = p1(self.X)
        m1.fit(x, self.y)
        m2.fit(x, self.y)
        r1 = m1.predict(p1(self.test_data))
        r2 = m2.predict(p1(self.test_data))
        manual_result = [r1, r2]

        assert len(pipeline_result) == len(manual_result)
        assert np.array_equal(pipeline_result[0], manual_result[0])
        assert np.array_equal(pipeline_result[1], manual_result[1])



from abc import ABC
from abc import abstractmethod

from accelera.src.utils.array_utils import convert_to_array


class BaseMetric(ABC):
    def __init__(
        self,
        metric_name,
        metric,
        y_true=None,
        X=None,
        plot_func=None,
        labels_name=None,
        headers_name=None,
        **params,
    ):
        self.metric = metric
        self.params = params
        self.metric_name = metric_name
        self.y_true = convert_to_array(y_true)
        self.X = convert_to_array(X)
        self.plot_func = plot_func
        self.labels_name = labels_name
        self.headers_name = headers_name

    @abstractmethod
    def execute(self, y_pred):
        pass



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from accelera.src.benchmark.backend import config as config
from accelera.src.benchmark.backend import schemas as schemas
from accelera.src.benchmark.backend import validations as validations



import pandas as pd

from accelera.src.accelera_pipe.core.report_base import ReportBase


class TabularPreprocessingReport(ReportBase):
    def __init__(self, folderpath, report_data, text_based=False):
        super().__init__(folderpath)
        self.content = ""
        self.report_data = report_data
        self.data_overview = self.report_data["data_overview"]
        self.drop_duplicates = self.report_data["drop_duplicates"]
        self.split = self.report_data["split"]
        self.drop_columns = self.report_data["drop_columns"]
        self.graphs = self.report_data["graphs"]
        self.preprocessing = self.report_data["preprocessing"]
        self.after_preprocessing = self.report_data["after_preprocessing"]
        self.text_based = text_based

    def show_data_heads(self, obj, field_name, name="dataset"):
        self.content += f"""<h3>First 5 rows of the {name}:</h3>\n
        {obj[field_name].to_html(index=False)}"""

    def show_info(self, obj):
        self.content += "<h3>Data Information:</h3>\n"
        self.content += f"<pre>{obj['info']}</pre>\n"

    def show_numeric_statis(self, obj):
        if obj["numerical_describe"] is not None:
            self.content += "<h3>Numerical Statistics:</h3>\n"
            self.content += f"{obj['numerical_describe'].to_html()}\n"

    def show_categoric_statis(self, obj):
        if obj["categorical_describe"] is not None:
            self.content += "<h3>Categorical Statistics:</h3>\n"
            self.content += f"{obj['categorical_describe'].to_html()}\n"

    def show_nulls(self, obj):
        self.content += "<h3>Missing Values:</h3>\n"
        missing_df = obj["missing_values"].to_frame(name="Missing Values")
        self.content += f"{missing_df.to_html()}\n"

    def show_dupplicats(self, obj):
        self.content += "<h3>Duplicates:</h3>\n"
        self.content += (
            f"<p> number of duplicates rows: {obj['duplicates_sum']}</p>\n"
        )
        self.content += "<p> Percentage of duplicates "
        self.content += f"rows: {obj['duplicates_percentage']} %</p>\n"

    def show_shape(self, obj):
        self.content += "<h3>Data Shape:</h3>\n"
        self.content += f"<p> {obj['shape']}</p> \n"

    def show_data_overview(self):
        self.content += "<div>\n"
        self.content += "<h2>Data Overview</h2>\n"
        self.show_data_heads(self.data_overview, "data_head")
        self.content += "</div>\n"
        self.content += "<div>\n"
        self.content += "<h2>Data Overview After lowering dataset</h2>\n"
        self.show_data_heads(self.data_overview, "lower_data_head")
        self.show_info(self.data_overview)
        self.show_shape(self.data_overview)
        self.show_numeric_statis(self.data_overview)
        self.show_categoric_statis(self.data_overview)
        self.show_nulls(self.data_overview)
        self.show_dupplicats(self.data_overview)
        self.content += "</div>\n"

    def show_drop_duplicates(self):
        self.content += "<div>\n"
        self.content += "<h2> After Drop Duplicates</h2>\n"
        self.show_shape(self.drop_duplicates)
        self.show_dupplicats(self.drop_duplicates)
        self.content += "</div>\n"

    def show_split(self):
        self.content += "<div>\n"
        self.content += "<h2> Train / Validation Split</h2>\n"
        self.content += "<h3> Val Size</h3>\n"
        self.content += f"<p>{self.split['val_size']}</p>\n"
        self.content += "<h3>Training set</h3>\n"
        self.content += f"X_train shape : {self.split['X_train_shape']}</p>\n"
        self.content += f"y_train shape : {self.split['y_train_shape']}</p>\n"
        self.content += "<h3>validation set</h3>\n"
        self.content += f"X_val shape : {self.split['X_val_shape']}</p>\n"
        self.content += f"y_val shape : {self.split['y_val_shape']}</p>\n"
        self.content += "</div>\n"

    def show_drop_col(self):
        self.content += "<div>\n"
        self.content += "<h2>Drop Columns</h2>\n"

        if self.drop_columns["col_drop"]:
            self.content += "<h3>Columns Removed & Justifications</h3>\n"
            self.content += "<table >\n"
            self.content += "<tr><th>Column</th><th>Reason</th></tr>\n"
            for col, reason in self.drop_columns["col_drop"].items():
                self.content += f"<tr><td>{col}</td><td>{reason}</td></tr>\n"
            self.content += "</table>\n"
            self.show_data_heads(self.drop_columns, "X_trian_head", "X train")
            self.show_data_heads(self.drop_columns, "X_val_head", "X validation")

        else:
            self.content += "<p>No columns were dropped</p>\n"
        self.content += "</div>\n"

    def show_preprocessing(self):
        self.content += "<div>\n"
        self.content += "<h2>Preprocessing</h2>\n"
        self.content += "<table>\n"

        self.content += (
            "<tr><th>Column</th><th>Predicted Type</th>"
            "<th>Preprocessing Steps</th></tr>\n"
        )

        for item in self.preprocessing:
            col_name, col_type, col_preprocessing = (
                item["col_name"],
                item["col_type"].title(),
                item["col_preprocessing"],
            )
            self.content += f"<tr><td>{col_name}</td>"
            self.content += f"<td>{col_type}</td>"
            self.content += "<td>"
            for i, step in enumerate(col_preprocessing):
                self.content += f"{step.title()} "
                if i != len(col_preprocessing) - 1:
                    self.content += "-> "
            self.content += "</td>"
            self.content += "</tr>\n"

        self.content += "</table>\n"
        self.content += "</div>\n"

    def show_after_preprocessing(self):
        self.content += "<div>\n"
        self.content += "<h2>After Preprocessing</h2>\n"
        training_df = pd.concat(
            [
                self.after_preprocessing["X_train_processed"],
                self.after_preprocessing["y_train_processed"],
            ],
            axis=1,
        )
        val_df = pd.concat(
            [
                self.after_preprocessing["X_val_processed"],
                self.after_preprocessing["y_val_processed"],
            ],
            axis=1,
        )
        self.after_preprocessing["training_df"] = training_df
        self.after_preprocessing["val_df"] = val_df
        self.show_data_heads(
            self.after_preprocessing,
            "training_df",
            "Training After Preprocessing Head",
        )
        self.show_data_heads(
            self.after_preprocessing,
            "val_df",
            "Validation After Preprocessing Head",
        )
        self.content += "</div>\n"

    def execute(self):
        self.show_data_overview()
        self.show_drop_duplicates()
        self.show_split()
        if not self.text_based:
            self.show_drop_col()
        self.show_graphs(self.graphs)
        self.show_preprocessing()
        self.show_after_preprocessing()
        full_content = self.start_content + self.content + self.end_content
        self.create_html_file(full_content)



import shutil
import tempfile

import numpy as np
import pytest
from PIL import Image

from accelera.src.automl.core.segmentation_image_training_preprocessing import (
    SegmentationImageTrainingPreprocessing,
)
from accelera.src.utils.preprocessing import check_path_exists


class TestSegmentationImageTrainingPreprocessing:
    @pytest.fixture(autouse=True)
    def temp_folder(self):
        self.temp_dir = tempfile.mkdtemp()
        yield self.temp_dir
        shutil.rmtree(self.temp_dir)

    def test_constractor(self, tmp_path):
        training_folder_images = tmp_path / "training_folder_images"
        training_folder_images.mkdir()
        training_folder_masks = tmp_path / "training_folder_masks"
        training_folder_masks.mkdir()

        validation_folder_images = tmp_path / "validation_folder_images"
        validation_folder_images.mkdir()
        validation_folder_masks = tmp_path / "validation_folder_masks"
        validation_folder_masks.mkdir()
        with pytest.raises(
            ValueError, match="Training folder masks must be not null"
        ):
            SegmentationImageTrainingPreprocessing(
                training_folder_images=training_folder_images,
                training_folder_masks=None,
                folder_path=self.temp_dir,
            )

        with pytest.raises(
            ValueError,
            match="training folder images and training folder masks must be "
            "different",
        ):
            SegmentationImageTrainingPreprocessing(
                training_folder_images=training_folder_images,
                training_folder_masks=training_folder_images,
                folder_path=self.temp_dir,
            )

        with pytest.raises(ValueError, match=r".*does not exist"):
            SegmentationImageTrainingPreprocessing(
                training_folder_images=training_folder_images,
                training_folder_masks="invalid",
                folder_path=self.temp_dir,
            )

        with pytest.raises(
            ValueError, match=r"you must add binary_mask_threshold.*"
        ):
            SegmentationImageTrainingPreprocessing(
                training_folder_images=training_folder_images,
                training_folder_masks=training_folder_masks,
                folder_path=self.temp_dir,
                binary_mask_threshold=None,
            )
        with pytest.raises(
            ValueError,
            match=r"binary_mask_threshold must be integer between 0 and 255",
        ):
            SegmentationImageTrainingPreprocessing(
                training_folder_images=training_folder_images,
                training_folder_masks=training_folder_masks,
                folder_path=self.temp_dir,
                binary_mask_threshold="invalid",
            )
        with pytest.raises(
            ValueError,
            match=r"binary_mask_threshold must be integer between 0 and 255",
        ):
            SegmentationImageTrainingPreprocessing(
                training_folder_images=training_folder_images,
                training_folder_masks=training_folder_masks,
                folder_path=self.temp_dir,
                binary_mask_threshold=-1,
            )
        with pytest.raises(
            ValueError, match=r"Validation folder masks must be not null"
        ):
            SegmentationImageTrainingPreprocessing(
                training_folder_images=training_folder_images,
                training_folder_masks=training_folder_masks,
                validation_folder_images=validation_folder_images,
                validation_folder_masks=None,
                folder_path=self.temp_dir,
                binary_mask_threshold=0,
            )
            with pytest.raises(ValueError, match=r".*does not exist"):
                SegmentationImageTrainingPreprocessing(
                    training_folder_images=training_folder_images,
                    training_folder_masks=training_folder_masks,
                    validation_folder_images=validation_folder_images,
                    validation_folder_masks="invalid",
                    folder_path=self.temp_dir,
                )
            with pytest.raises(
                ValueError,
                match="training folder images and training folder masks "
                "must be different",
            ):
                SegmentationImageTrainingPreprocessing(
                    training_folder_images=training_folder_images,
                    training_folder_masks=training_folder_images,
                    validation_folder_images=validation_folder_images,
                    validation_folder_masks=validation_folder_images,
                    folder_path=self.temp_dir,
                )
            assert check_path_exists(self.temp_dir, "data_info.pkl")

    def test_data_preparing(self, tmp_path):
        training_folder_images = tmp_path / "training_folder_images"
        training_folder_images.mkdir()
        training_folder_masks = tmp_path / "training_folder_masks"
        training_folder_masks.mkdir()

        validation_folder_images = tmp_path / "validation_folder_images"
        validation_folder_images.mkdir()
        validation_folder_masks = tmp_path / "validation_folder_masks"
        validation_folder_masks.mkdir()
        for i in range(5):
            (training_folder_images / f"invalid{i}.png").touch()
            (training_folder_masks / f"invalid_mask{i}.png").touch()

        with pytest.raises(
            ValueError, match="no matches between masks and images names"
        ):
            preprocessor = SegmentationImageTrainingPreprocessing(
                training_folder_images=training_folder_images,
                training_folder_masks=training_folder_masks,
                folder_path=self.temp_dir,
            )
            preprocessor.common_preprocessing()
        for i in range(5):
            (training_folder_images / f"invalid{i}.png").touch()
            (training_folder_masks / f"invalid{i}.png").touch()
        with pytest.raises(ValueError, match="There is no valid path"):
            SegmentationImageTrainingPreprocessing(
                training_folder_images=training_folder_images,
                training_folder_masks=training_folder_masks,
                folder_path=self.temp_dir,
            ).common_preprocessing()

        for i in range(5):
            img = Image.fromarray(
                np.random.randint(0, 256, (225, 225, 3), dtype=np.uint8)
            )
            mask = Image.fromarray(
                np.random.randint(0, 256, (225, 225), dtype=np.uint8)
            )
            img.save(training_folder_images / f"img_{i}.png")
            mask.save(training_folder_masks / f"img_{i}.png")
        preprocessor = SegmentationImageTrainingPreprocessing(
            training_folder_images=training_folder_images,
            training_folder_masks=training_folder_masks,
            folder_path=self.temp_dir,
        )
        invalid_images, invalid_masks = [], []
        training_images, training_masks = preprocessor.data_preparing(
            images_folder_path=training_folder_images,
            masks_folder_path=training_folder_masks,
            invalid_masks_paths=invalid_masks,
            invalid_images_paths=invalid_images,
        )
        assert len(invalid_images) == 5
        assert len(invalid_masks) == 5
        assert len(training_images) == 5
        assert len(training_masks) == 5

    def test_data_overview(self, tmp_path):
        training_folder_images = tmp_path / "training_folder_images"
        training_folder_images.mkdir()
        training_folder_masks = tmp_path / "training_folder_masks"
        training_folder_masks.mkdir()

        validation_folder_images = tmp_path / "validation_folder_images"
        validation_folder_images.mkdir()
        validation_folder_masks = tmp_path / "validation_folder_masks"
        validation_folder_masks.mkdir()
        for i in range(5):
            (training_folder_images / f"invalid{i}.png").touch()
            (training_folder_masks / f"invalid{i}.png").touch()
            (validation_folder_images / f"invalid{i}.png").touch()
            (validation_folder_masks / f"invalid{i}.png").touch()

        for i in range(5):
            img = Image.fromarray(
                np.random.randint(0, 256, (225, 225, 3), dtype=np.uint8)
            )
            mask = Image.fromarray(
                np.random.randint(0, 256, (225, 225), dtype=np.uint8)
            )
            img.save(training_folder_images / f"img_{i}.png")
            mask.save(training_folder_masks / f"img_{i}.png")
            img.save(validation_folder_images / f"img_{i}.png")
            mask.save(validation_folder_masks / f"img_{i}.png")

        preprocessor = SegmentationImageTrainingPreprocessing(
            training_folder_images=training_folder_images,
            training_folder_masks=training_folder_masks,
            validation_folder_images=validation_folder_images,
            validation_folder_masks=validation_folder_masks,
            folder_path=self.temp_dir,
        )
        preprocessor.common_preprocessing()
        assert "training_folder" in preprocessor.report_data["data_overview"]
        assert "validation_folder" in preprocessor.report_data["data_overview"]
        assert (
            preprocessor.report_data["data_overview"]["training_folder"][
                "images_len"
            ]
            == 5
        )
        assert (
            preprocessor.report_data["data_overview"]["training_folder"][
                "invalid_len"
            ]
            == 5
        )
        assert (
            preprocessor.report_data["data_overview"]["validation_folder"][
                "images_len"
            ]
            == 5
        )
        assert (
            preprocessor.report_data["data_overview"]["validation_folder"][
                "invalid_len"
            ]
            == 5
        )

    def test_splitting(self, tmp_path):
        training_folder_images = tmp_path / "training_folder_images"
        training_folder_images.mkdir()
        training_folder_masks = tmp_path / "training_folder_masks"
        training_folder_masks.mkdir()

        validation_folder_images = tmp_path / "validation_folder_images"
        validation_folder_images.mkdir()
        validation_folder_masks = tmp_path / "validation_folder_masks"
        validation_folder_masks.mkdir()
        for i in range(10):
            img = Image.fromarray(
                np.random.randint(0, 256, (225, 225, 3), dtype=np.uint8)
            )
            mask = Image.fromarray(
                np.random.randint(0, 256, (225, 225), dtype=np.uint8)
            )
            img.save(training_folder_images / f"img_{i}.png")
            mask.save(training_folder_masks / f"img_{i}.png")
            img.save(validation_folder_images / f"img_{i}.png")
            mask.save(validation_folder_masks / f"img_{i}.png")

        preprocessor = SegmentationImageTrainingPreprocessing(
            training_folder_images=training_folder_images,
            training_folder_masks=training_folder_masks,
            folder_path=self.temp_dir,
        )
        preprocessor.common_preprocessing()
        assert len(preprocessor.training_paths) == 10
        assert len(preprocessor.training_labels) == 10
        assert preprocessor.validation_paths is None
        assert preprocessor.validation_labels is None
        preprocessor = SegmentationImageTrainingPreprocessing(
            training_folder_images=training_folder_images,
            training_folder_masks=training_folder_masks,
            folder_path=self.temp_dir,
            split_training=True,
        )
        preprocessor.common_preprocessing()
        assert len(preprocessor.training_paths) == 8
        assert len(preprocessor.training_labels) == 8
        assert len(preprocessor.validation_paths) == 2
        assert len(preprocessor.validation_labels) == 2

    def test_get_loaders(self, tmp_path):
        training_folder_images = tmp_path / "training_folder_images"
        training_folder_images.mkdir()
        training_folder_masks = tmp_path / "training_folder_masks"
        training_folder_masks.mkdir()

        validation_folder_images = tmp_path / "validation_folder_images"
        validation_folder_images.mkdir()
        validation_folder_masks = tmp_path / "validation_folder_masks"
        validation_folder_masks.mkdir()
        for i in range(10):
            img = Image.fromarray(
                np.random.randint(0, 256, (225, 225, 3), dtype=np.uint8)
            )
            mask = Image.fromarray(
                np.random.randint(0, 256, (225, 225), dtype=np.uint8)
            )
            img.save(training_folder_images / f"img_{i}.png")
            mask.save(training_folder_masks / f"img_{i}.png")
            img.save(validation_folder_images / f"img_{i}.png")
            mask.save(validation_folder_masks / f"img_{i}.png")

        preprocessor = SegmentationImageTrainingPreprocessing(
            training_folder_images=training_folder_images,
            training_folder_masks=training_folder_masks,
            folder_path=self.temp_dir,
        )
        preprocessor.common_preprocessing()
        assert preprocessor.training_loader is not None
        assert preprocessor.validation_loader is None
        preprocessor = SegmentationImageTrainingPreprocessing(
            training_folder_images=training_folder_images,
            training_folder_masks=training_folder_masks,
            folder_path=self.temp_dir,
            split_training=True,
        )
        preprocessor.common_preprocessing()
        assert preprocessor.training_loader is not None
        assert preprocessor.validation_loader is not None
        preprocessor = SegmentationImageTrainingPreprocessing(
            training_folder_images=training_folder_images,
            training_folder_masks=training_folder_masks,
            validation_folder_images=validation_folder_images,
            validation_folder_masks=validation_folder_masks,
            folder_path=self.temp_dir,
        )
        preprocessor.common_preprocessing()
        assert preprocessor.training_loader is not None
        assert preprocessor.validation_loader is not None



import shutil
import tempfile

import pandas as pd
import pytest
from sklearn.preprocessing import LabelEncoder

from accelera.src.automl.core.classical_testing_preprocessing import (
    ClassicalTestingPreprocessing,
)
from accelera.src.utils.preprocessing import drop_columns
from accelera.src.utils.preprocessing import save_pickle


class TestClassicalTestingPreprocessing:
    @pytest.fixture(autouse=True)
    def initializetion(self):
        self.temp_dir = tempfile.mkdtemp()
        data_columns = ["feature1", "feature2", "target"]
        col_drop = {"feature2": "high missing values"}
        target_info = {
            "col_name": "target",
            "problem_type": "classification",
            "mode": "A",
            "median": None,
        }
        self.df = pd.DataFrame(
            {
                "feature1": ["A", "b", "c"],
                "feature2": [None, None, None],
                "target": ["A", None, "C"],
            }
        )
        save_pickle(self.temp_dir, data_columns, "data_columns.pkl")
        save_pickle(self.temp_dir, col_drop, "col_drop.pkl")
        save_pickle(self.temp_dir, target_info, "target_info.pkl")
        yield self.temp_dir
        shutil.rmtree(self.temp_dir)

    def test_classical_errors_checks_testing(self):
        with pytest.raises(ValueError):
            ClassicalTestingPreprocessing(df=None, folder_path=self.temp_dir)
        with pytest.raises(ValueError):
            ClassicalTestingPreprocessing(df=self.df, folder_path=None)
        with pytest.raises(ValueError):
            ClassicalTestingPreprocessing(df=self.df, folder_path=self.temp_dir)
        save_pickle(self.temp_dir, {}, "target_preprocessor.pkl")

        with pytest.raises(ValueError):
            ClassicalTestingPreprocessing(df=self.df, folder_path=self.temp_dir)
        save_pickle(self.temp_dir, {}, "training_preprocessor.pkl")
        target_info_invalid = {
            "problem_type": "classification",
            "mode": "A",
            "median": None,
        }
        save_pickle(self.temp_dir, target_info_invalid, "target_info.pkl")
        with pytest.raises(ValueError):
            ClassicalTestingPreprocessing(df=self.df, folder_path=self.temp_dir)
        target_info_invalid = {
            "col_name": "target",
            "problem_type": "invalid_problem_type",
            "mode": "A",
            "median": None,
        }
        save_pickle(self.temp_dir, target_info_invalid, "target_info.pkl")

        with pytest.raises(ValueError):
            ClassicalTestingPreprocessing(df=self.df, folder_path=self.temp_dir)
        with pytest.raises(ValueError):
            ClassicalTestingPreprocessing(
                df=self.df.drop(columns=["feature1"]), folder_path=self.temp_dir
            )

    def test_classical_setup_test(self):
        save_pickle(self.temp_dir, {}, "target_preprocessor.pkl")
        save_pickle(self.temp_dir, {}, "training_preprocessor.pkl")

        target_info = {
            "col_name": "target",
            "problem_type": "classification",
            "mode": "A",
            "median": None,
        }
        save_pickle(self.temp_dir, target_info, "target_info.pkl")

        lower_df = self.df.copy()
        for col in lower_df.select_dtypes(include="object").columns:
            lower_df[col] = lower_df[col].apply(
                lambda x: x.lower() if isinstance(x, str) else x
            )
        tp = ClassicalTestingPreprocessing(
            df=self.df.drop(columns=["target"]), folder_path=self.temp_dir
        )
        assert tp.features_only is True
        assert list(tp.X_test.columns) == ["feature1", "feature2"]
        assert tp.data_columns == ["feature1", "feature2"]
        assert tp.y_test is None
        assert tp.X_test.equals(lower_df.drop(columns=["target"]))
        tp = ClassicalTestingPreprocessing(df=self.df, folder_path=self.temp_dir)
        assert tp.features_only is False
        assert list(tp.X_test.columns) == ["feature1", "feature2"]
        assert tp.data_columns == ["feature1", "feature2", "target"]
        assert tp.y_test.equals(lower_df["target"])
        assert tp.X_test.equals(lower_df.drop(columns=["target"]))

    def test_classical_drop_columns_test(self):
        save_pickle(self.temp_dir, {}, "target_preprocessor.pkl")
        save_pickle(self.temp_dir, {}, "training_preprocessor.pkl")

        target_info = {
            "col_name": "target",
            "problem_type": "classification",
            "mode": "A",
            "median": None,
        }
        save_pickle(self.temp_dir, target_info, "target_info.pkl")

        tp = ClassicalTestingPreprocessing(df=self.df, folder_path=self.temp_dir)
        drop_columns(tp.X_test, tp.col_drop)
        assert list(tp.X_test.columns) == ["feature1"]

    def test_classical_lower_data_test(self):
        save_pickle(self.temp_dir, {}, "target_preprocessor.pkl")
        save_pickle(self.temp_dir, {}, "training_preprocessor.pkl")

        target_info = {
            "col_name": "target",
            "problem_type": "classification",
            "mode": "A",
            "median": None,
        }
        save_pickle(self.temp_dir, target_info, "target_info.pkl")

        tp = ClassicalTestingPreprocessing(df=self.df, folder_path=self.temp_dir)
        assert tp.X_test["feature1"].tolist() == ["a", "b", "c"]
        assert tp.y_test.tolist() == ["a", None, "c"]

    def test_classical_target_preprocessing_test(self):
        target_df = pd.DataFrame({"target": ["a", "b", "c"]})
        label_encoder = LabelEncoder()
        label_encoder.fit(target_df["target"])

        save_pickle(self.temp_dir, label_encoder, "target_preprocessor.pkl")
        save_pickle(self.temp_dir, {}, "training_preprocessor.pkl")

        target_info = {
            "col_name": "target",
            "problem_type": "classification",
            "mode": "b",
            "median": None,
        }
        save_pickle(self.temp_dir, target_info, "target_info.pkl")

        tp = ClassicalTestingPreprocessing(df=self.df, folder_path=self.temp_dir)
        tp.target_preprocessing()
        assert tp.y_test.tolist() == [0, 1, 2]



from torch.utils.data import DataLoader

from accelera.src.automl.core.preprocessing_base import PreprocessingBase
from accelera.src.automl.core.segmentation_image_dataset import (
    SegmentationImageDataset,
)
from accelera.src.utils.preprocessing import check_path_exists
from accelera.src.utils.preprocessing import collect_function_segmentation
from accelera.src.utils.preprocessing import is_valid_image
from accelera.src.utils.preprocessing import load_pickle


class SegmentationImageTestingPreprocessing(PreprocessingBase):
    def __init__(
        self,
        image_paths,
        image_masks=None,
        folder_path=None,
        batch_size=4,
    ):
        super().__init__(folder_path)
        self.image_paths = image_paths
        self.batch_size = batch_size
        self.image_masks = image_masks
        self.valid_images = []
        self.valid_masks = []
        self.invalid_images = []
        if self.image_paths is None:
            raise ValueError("Image paths must be list of paths not none")
        if not isinstance(self.image_paths, list):
            raise ValueError("Image paths must be list of paths")
        if len(self.image_paths) == 0:
            raise ValueError("Image paths is empty list")
        if self.image_masks is not None and not isinstance(self.image_masks, list):
            raise ValueError("masks must be list of masks paths")
        if self.image_masks is not None and len(self.image_masks) != len(
            self.image_paths
        ):
            raise ValueError("image paths length must equal masks length")
        for i, path in enumerate(self.image_paths):
            if is_valid_image(path):
                if self.image_masks is not None:
                    if is_valid_image(self.image_masks[i]):
                        self.valid_images.append(path)
                        self.valid_masks.append(self.image_masks[i])
                    else:
                        self.invalid_images.append(path)
                else:
                    self.valid_images.append(path)
            else:
                self.invalid_images.append(path)
        if len(self.valid_images) == 0:
            raise ValueError("There is no valid image exists")

        check_path_exists(self.folder_path, "data_info.pkl")
        info = load_pickle(self.folder_path, "data_info.pkl")
        self.image_size = info["image_size"]
        self.binary_mask_threshold = info["binary_mask_threshold"]

    def common_preprocessing(self):
        self.valid_masks = None if len(self.valid_masks) == 0 else self.valid_masks
        dataset = SegmentationImageDataset(
            self.valid_images,
            self.valid_masks,
            self.image_size,
            self.binary_mask_threshold,
            augment=False,
        )
        testing_loader = DataLoader(
            dataset,
            batch_size=self.batch_size,
            shuffle=False,
            collate_fn=collect_function_segmentation,
        )
        return testing_loader, self.invalid_images



import inspect
import logging
import os
import shutil
import sys

import sklearn.metrics as metrics

from accelera.src.accelera_pipe.wrappers.supervised_metric import SupervisedMetric
from accelera.src.accelera_pipe.wrappers.unsupervised_metric import (
    UnSupervisedMetric,
)

try:
    from graph import serialize_graph
except ImportError as e:
    raise ImportError(
        "The 'graph' C++ module could not be imported. "
        "Please ensure it is built and available in your PYTHONPATH."
    ) from e

interactive = True


def print_msg(message, line_break=True, level="info"):
    global interactive
    if interactive:
        if line_break:
            sys.stdout.write(message + "\n")
        else:
            sys.stdout.write(message)
        sys.stdout.flush()
    else:
        if level == "info":
            logging.info(message)
        elif level == "warning":
            logging.warning(message)
        elif level == "error":
            logging.error(message)
        else:
            logging.debug(message)


def get_metric_object(
    metric_name: str,
):
    if metric_name == "":
        return None
    metric_func = getattr(metrics, metric_name, None)
    return metric_func


def get_correct_metric_class(
    metric_name,
    metric,
    y_true=None,
    plot_func=None,
    labels_name=None,
    headers_name=None,
    **params,
):
    signature = inspect.signature(metric)
    parameters = list(signature.parameters.keys())
    has_true_labels = any(param in parameters for param in ["y_true", "labels_true"])
    has_predictions = any(
        param in parameters
        for param in ["y_pred", "y_score", "y_proba", "labels_pred"]
    )
    supervised = has_true_labels and has_predictions
    unsupervised = "X" in parameters and "labels" in parameters
    if supervised:
        return SupervisedMetric(
            metric_name,
            metric,
            y_true,
            plot_func,
            labels_name,
            headers_name,
            **params,
        )
    elif unsupervised:
        print_msg(
            f"Using unsupervised metric '{metric_name}' "
            "doesn't require y_true. "
            "y_true will be ignored.",
            level="warning",
        )
        return UnSupervisedMetric(
            metric_name, metric, plot_func, labels_name, headers_name, **params
        )
    else:
        return None


def check_path_exist(path):
    if os.path.exists(path):
        return True
    else:
        return False


def create_folder(folder_path):
    if check_path_exist(folder_path):
        shutil.rmtree(folder_path)

    os.makedirs(folder_path, exist_ok=True)


def get_instance_type(instance) -> str:
    def check_method_patterns():
        has_predict = hasattr(instance, "predict")
        has_transform = hasattr(instance, "transform")
        has_fit_predict = hasattr(instance, "fit_predict")
        has_predict_proba = hasattr(instance, "predict_proba")
        has_decision_function = hasattr(instance, "decision_function")
        has_fit = hasattr(instance, "fit")

        supervised_score = 0
        if has_predict_proba:
            supervised_score += 3
        if has_decision_function:
            supervised_score += 2
        if has_predict and not has_transform:
            supervised_score += 2

        unsupervised_score = 0
        if has_fit_predict:
            unsupervised_score += 2
        if has_transform and not has_predict_proba:
            unsupervised_score += 2
        if hasattr(instance, "cluster_centers_"):
            unsupervised_score += 3
        if hasattr(instance, "components_"):
            unsupervised_score += 2

        # Check for preprocessing patterns
        if has_transform and has_fit and not has_predict and not has_fit_predict:
            # This is likely a preprocessor/transformer
            # Check if it needs labels (supervised preprocessing)
            if hasattr(instance, "score_func") or hasattr(instance, "scores_"):
                supervised_score += 2
            elif hasattr(instance, "k") or hasattr(instance, "percentile"):
                supervised_score += 1
            else:
                unsupervised_score += 1

        if supervised_score > unsupervised_score:
            return "supervised"
        elif unsupervised_score > supervised_score:
            return "unsupervised"
        return None

    def check_class_name():
        class_name = instance.__class__.__name__.lower()

        supervised_patterns = [
            "classifier",
            "regressor",
            "svm",
            "svc",
            "svr",
            "logistic",
            "linear",
            "ridge",
            "lasso",
            "tree",
            "forest",
            "boost",
            "bagging",
            "voting",
            "neural",
            # Feature selection (supervised preprocessing)
            "selectkbest",
            "selectpercentile",
            "selectfpr",
            "selectfdr",
            "selectfwe",
            "rfe",
            "rfecv",
            "variancethreshold",
            # Target-based preprocessing
            "targetencoder",
            "ordinalencoder",
        ]

        unsupervised_patterns = [
            "kmeans",
            "cluster",
            "pca",
            "ica",
            "tsne",
            "dbscan",
            "isolation",
            "anomaly",
            "outlier",
            "decomposition",
            # Unsupervised preprocessing
            "standardscaler",
            "minmaxscaler",
            "robustscaler",
            "normalizer",
            "quantiletransformer",
            "powertransformer",
            "polynomialfeatures",
            "onehotencoder",
            "labelencoder",
            "imputer",
            "simpleimputer",
        ]

        for pattern in supervised_patterns:
            if pattern in class_name:
                return "supervised"

        for pattern in unsupervised_patterns:
            if pattern in class_name:
                return "unsupervised"
        return None

    def check_module_origin():
        module = instance.__class__.__module__
        if module:
            if "cluster" in module or "decomposition" in module:
                return "unsupervised"
            elif (
                "ensemble" in module
                or "linear_model" in module
                or "svm" in module
                or "feature_selection" in module
            ):
                return "supervised"
            elif "preprocessing" in module:
                # Most sklearn preprocessing is unsupervised (scalers, encoders)
                # but feature selection is supervised
                if "feature_selection" in module:
                    return "supervised"
                else:
                    return "unsupervised"
        return None

    def check_model_attributes():
        supervised_attrs = [
            "classes_",
            "n_classes_",
            "coef_",
            "intercept_",
            "feature_importances_",
            "estimators_",
            # Feature selection attributes
            "scores_",
            "pvalues_",
            "support_",
            "ranking_",
        ]

        unsupervised_attrs = [
            "cluster_centers_",
            "labels_",
            "inertia_",
            "components_",
            "explained_variance_",
            "singular_values_",
            "n_clusters",
            # Preprocessing attributes
            "scale_",
            "mean_",
            "var_",
            "min_",
            "max_",
            "data_min_",
            "data_max_",
            "categories_",
        ]

        supervised_count = sum(
            1 for attr in supervised_attrs if hasattr(instance, attr)
        )
        unsupervised_count = sum(
            1 for attr in unsupervised_attrs if hasattr(instance, attr)
        )

        if supervised_count > unsupervised_count:
            return "supervised"
        elif unsupervised_count > supervised_count:
            return "unsupervised"
        return None

    strategies = [
        check_method_patterns,
        check_class_name,
        check_module_origin,
        check_model_attributes,
    ]

    votes = {"supervised": 0, "unsupervised": 0}

    for strategy in strategies:
        try:
            result = strategy()
            if result in votes:
                votes[result] += 1
        except Exception:
            continue

    if votes["supervised"] > votes["unsupervised"]:
        return "supervised"
    elif votes["unsupervised"] > votes["supervised"]:
        return "unsupervised"
    else:
        return "supervised"


def execute_fit(instance, x, y):
    instance_type = get_instance_type(instance)
    if instance_type == "supervised":
        instance.fit(x, y)
    elif instance_type == "unsupervised":
        instance.fit(x)
    return instance


def serialize(pipeline, filepath):
    serialize_graph(pipeline._PipelineBase__graph, filepath)


def load(directory):
    raise NotImplementedError("Loading from directory is not implemented yet.")



import random

from PIL import ImageEnhance
from torch.utils.data import Dataset


class ImageDataset(Dataset):
    def __init__(
        self,
        image_paths,
        labels=None,
        image_size=(224, 224),
        augment=True,
        augmentation_probability=0.5,
        horizontal_flip=True,
        vertical_flip=True,
        rotation=True,
        rotation_angle=30,
        brightness=True,
        brightness_factors=(0.8, 1.2),
        contrast=True,
        contrast_factors=(0.8, 1.2),
    ):
        self.image_paths = image_paths
        self.labels = labels
        self.image_size = image_size
        self.augment = augment
        self.horizontal_flip = horizontal_flip
        self.vertical_flip = vertical_flip
        self.augmentation_probability = augmentation_probability
        self.rotation = rotation
        self.rotation_angle = rotation_angle
        self.brightness = brightness
        self.brightness_factors = brightness_factors
        self.contrast = contrast
        self.contrast_factors = contrast_factors

    def __len__(self):
        return len(self.image_paths)

    def random_brightness(self, img):
        if self.brightness and random.random() < self.augmentation_probability:
            random_factor = random.uniform(*self.brightness_factors)
            return ImageEnhance.Brightness(img).enhance(random_factor)
        return img

    def random_contrast(self, img):
        if self.contrast and random.random() < self.augmentation_probability:
            random_factor = random.uniform(*self.contrast_factors)
            return ImageEnhance.Contrast(img).enhance(random_factor)
        return img



import os

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from PIL import Image

from accelera.src.automl.wrappers.graph_base import GraphBase


class DisplaySampleImagesSegmentation(GraphBase):
    def __init__(
        self,
        paths,
        masks,
        folder_path,
        n_sample=4,
        title="",
        file_name="",
    ):
        super().__init__(folder_path)
        self.title = title
        self.file_name = file_name
        self.n_sample = n_sample
        self.df = pd.DataFrame({"paths": paths, "masks": masks})
        self.sample = self.df.sample(
            n=min(self.n_sample, len(paths)), random_state=42
        )

    def build_graph(self):
        cols = len(self.sample)
        fig, ax = plt.subplots(2, cols, figsize=(4 * cols, 4 * 2))
        fig.suptitle(self.title, fontsize=20)
        ax = np.atleast_2d(ax)
        for i, (image_path, mask_path) in enumerate(
            zip(self.sample["paths"], self.sample["masks"])
        ):
            ax[0][i].axis("off")
            img = np.array(Image.open(image_path).convert("RGB"))
            ax[0][i].imshow(img)
            ax[0][i].set_title(f"image {os.path.split(image_path)[-1]}")
            ax[1][i].axis("off")
            mask = plt.imread(mask_path)
            ax[1][i].imshow(mask, cmap="gray")

            ax[1][i].set_title(f"mask {os.path.split(mask_path)[-1]}")
        plt.tight_layout()
        plt.savefig(os.path.join(self.folder_path, f"{self.file_name}.png"))
        plt.close()



from torch.utils.data import DataLoader

from accelera.src.automl.core.classification_image_dataset import (
    ClassificationImageDataset,
)
from accelera.src.automl.core.preprocessing_base import PreprocessingBase
from accelera.src.utils.preprocessing import check_path_exists
from accelera.src.utils.preprocessing import collect_function
from accelera.src.utils.preprocessing import is_valid_image
from accelera.src.utils.preprocessing import load_pickle


class ClassificationImageTestingPreprocessing(PreprocessingBase):
    def __init__(
        self,
        image_paths,
        image_class_names=None,
        folder_path=None,
        batch_size=4,
    ):
        super().__init__(folder_path)
        self.image_paths = image_paths
        self.image_class_names = image_class_names
        self.batch_size = batch_size
        self.valid_images = []
        self.valid_images_class_names = []
        self.invalid_images = []
        if self.image_paths is None:
            raise ValueError("Image paths must be list of paths not none")

        if not isinstance(self.image_paths, list):
            raise ValueError("Image paths must be list of paths")
        if len(self.image_paths) == 0:
            raise ValueError("Image paths is empty list")
        if self.image_class_names is not None and not isinstance(
            self.image_class_names, list
        ):
            raise ValueError("Class names must be list of class names")
        if self.image_class_names is not None and len(self.image_class_names) != len(
            self.image_paths
        ):
            raise ValueError("Image paths length must equal class names length")
        for i, path in enumerate(self.image_paths):
            if is_valid_image(path):
                self.valid_images.append(path)
                if self.image_class_names is not None:
                    self.valid_images_class_names.append(self.image_class_names[i])
            else:
                self.invalid_images.append(path)
        if len(self.valid_images) == 0:
            raise ValueError("There is no valid image exists")

        check_path_exists(self.folder_path, "data_info.pkl")
        self.image_size = load_pickle(self.folder_path, "data_info.pkl")[
            "image_size"
        ]
        check_path_exists(self.folder_path, "class2label_mapping.pkl")
        self.class2label_mapping = load_pickle(
            self.folder_path, "class2label_mapping.pkl"
        )

    def common_preprocessing(self):
        labels = None
        if self.image_class_names is not None:
            labels = []
            for class_name in self.valid_images_class_names:
                if class_name not in self.class2label_mapping:
                    raise ValueError("this class name not in the training class")
                labels.append(self.class2label_mapping[class_name])

        dataset = ClassificationImageDataset(
            self.valid_images, labels, self.image_size, augment=False
        )
        testing_loader = DataLoader(
            dataset,
            batch_size=self.batch_size,
            shuffle=False,
            collate_fn=collect_function,
        )
        return testing_loader, self.invalid_images



from __future__ import annotations

import os
import re
import shutil
from typing import Any
from typing import Dict
from typing import List
from urllib.parse import urlparse

import pandas as pd
import requests
from platformdirs import user_cache_dir

from accelera.src.config import config
from accelera.src.utils.accelera_utils import print_msg


class DatasetRetriever:
    def __init__(self):
        self.is_connected = False
        self.cache_dir = None

    def _fetch_url_content(self, url: str) -> Dict[str, Any]:
        raw_url = url
        parsed = urlparse(url)

        if "drive.google.com" in parsed.netloc:
            raw_url = url

        try:
            response = requests.get(
                raw_url,
                headers={"User-Agent": "Mozilla/5.0"},
                timeout=config.REQUEST_TIMEOUT_S,
            )
            response.raise_for_status()

            return {
                "content": response.text,
                "source": raw_url,
            }

        except Exception as exc:
            return {"error": f"Failed to fetch page content: {exc}"}

    def available_datasets(
        self, url: str = config.DATASET_DRIVE_FOLDER_ENDPOINT
    ) -> List[str]:
        result = self._fetch_url_content(url)

        if "error" in result:
            return []

        html = result["content"]
        names = re.findall(r"[\w\-. ]+\.csv", html)

        if not names:
            raise ValueError("No datasets found")

        return list(set(name.replace("x22", "").split(".csv")[0] for name in names))

    def connect(self):
        self.is_connected = True
        self.cache_dir = user_cache_dir("accelera")
        os.makedirs(self.cache_dir, exist_ok=True)

    def retrieve_dataset(
        self,
        dataset_name: str,
        encoding: str = "utf-8",
        df: bool = False,
        url=None,
    ) -> pd.DataFrame:
        def get_df(dataset_path: str):
            return pd.read_csv(
                dataset_path,
                encoding=encoding,
                sep=None,
                engine="python",
                on_bad_lines="warn",
            )

        if not self.is_connected or not self.cache_dir:
            raise RuntimeError(
                "Dataset retriever is not connected. Call `connect()` first."
            )

        try:
            import gdown
        except ModuleNotFoundError as exc:
            raise ModuleNotFoundError(
                "Missing optional dependency 'gdown'. Run pip install gdown."
            ) from exc

        dataset_path = os.path.join(self.cache_dir, f"{dataset_name}.csv")
        if os.path.exists(dataset_path):
            if df:
                return get_df(dataset_path)
            return str(dataset_path)

        if url is None:
            if dataset_name not in config.DATASETS:
                raise KeyError(
                    f"Unknown dataset '{dataset_name}'. "
                    f"Available: {self.available_datasets()}"
                )

            gdown.download(
                id=config.DATASETS[dataset_name]["id"],
                output=dataset_path,
                quiet=False,
            )
        else:
            gdown.download(
                url=url,
                output=dataset_path,
                quiet=False,
            )

        print_msg(f"Downloaded dataset '{dataset_name}'", level="info")

        if df:
            return get_df(dataset_path)
        return str(dataset_path)

    def close(self):
        if not self.is_connected:
            raise RuntimeError("Dataset retriever is not connected")

        self.is_connected = False
        if self.cache_dir:
            shutil.rmtree(self.cache_dir)
        else:
            print_msg("Cache directory not found", level="warning")

        self.cache_dir = None


retriever = DatasetRetriever()



from sklearn.base import BaseEstimator
from sklearn.base import TransformerMixin


class CustomTransformer(BaseEstimator, TransformerMixin):
    def fit(self, X, y=None):
        raise NotImplementedError("CustomTransformer.fit is not implemented yet.")

    def transform(self, X):
        raise NotImplementedError(
            "CustomTransformer.transform is not implemented yet."
        )



import shutil
import tempfile

import numpy as np
import pytest
from PIL import Image

from accelera.src.automl.core.classification_image_training_preprocessing import (  # noqa: E501
    ClassificationImageTrainingPreprocessing,
)
from accelera.src.utils.preprocessing import check_path_exists


class TestClassificationImageTrainingPreprocessing:
    @pytest.fixture(autouse=True)
    def temp_folder(self):
        self.temp_dir = tempfile.mkdtemp()
        yield self.temp_dir
        shutil.rmtree(self.temp_dir)

    def test_constractor(self, tmp_path):
        training_folder = tmp_path / "training_folder"
        training_folder.mkdir()

        validation_folder = tmp_path / "validation_folder"
        validation_folder.mkdir()
        with pytest.raises(
            ValueError,
            match="Training Folder dosen't have any folder inside it ",
        ):
            ClassificationImageTrainingPreprocessing(
                training_folder_images=training_folder,
                folder_path=self.temp_dir,
            )

        (training_folder / "cats").mkdir()
        (training_folder / "dogs").mkdir()
        with pytest.raises(
            ValueError,
            match="Validation Folder dosen't have any folder inside it ",
        ):
            ClassificationImageTrainingPreprocessing(
                training_folder_images=training_folder,
                validation_folder_images=validation_folder,
                folder_path=self.temp_dir,
            )
        (validation_folder / "cats").mkdir()
        (validation_folder / "dogs").mkdir()
        (validation_folder / "birds").mkdir()
        with pytest.raises(
            ValueError, match=r"This category birds not in the training .*"
        ):
            ClassificationImageTrainingPreprocessing(
                training_folder_images=training_folder,
                validation_folder_images=validation_folder,
                folder_path=self.temp_dir,
            )

    def test_get_classes_mapping(self, tmp_path):
        training_folder = tmp_path / "training_folder"
        training_folder.mkdir()
        (training_folder / "cats").mkdir()
        (training_folder / "dogs").mkdir()
        preprocessor = ClassificationImageTrainingPreprocessing(
            training_folder_images=training_folder, folder_path=self.temp_dir
        )
        preprocessor.get_classes_mapping()
        assert len(preprocessor.class2label_mapping) == 2
        assert len(preprocessor.label2class_mapping) == 2
        assert "cats" in preprocessor.class2label_mapping
        assert "dogs" in preprocessor.class2label_mapping
        assert preprocessor.class2label_mapping["cats"] == 0
        assert preprocessor.class2label_mapping["dogs"] == 1
        assert preprocessor.label2class_mapping[0] == "cats"
        assert preprocessor.label2class_mapping[1] == "dogs"
        assert check_path_exists(self.temp_dir, "class2label_mapping.pkl")
        assert check_path_exists(self.temp_dir, "label2class_mapping.pkl")

    def test_data_preparing(self, tmp_path):
        training_folder = tmp_path / "training_folder"
        training_folder.mkdir()
        cats_folder = training_folder / "cats"
        cats_folder.mkdir()
        dogs_folder = training_folder / "dogs"
        dogs_folder.mkdir()
        for i in range(5):
            (cats_folder / f"invalid_cat_{i}.png").touch()
            (dogs_folder / f"invalid_dog_{i}.png").touch()
        with pytest.raises(ValueError, match="There is no valid path"):
            ClassificationImageTrainingPreprocessing(
                training_folder_images=training_folder,
                folder_path=self.temp_dir,
            ).common_preprocessing()

        for i in range(5):
            img = Image.fromarray(
                np.random.randint(0, 256, (225, 225, 3), dtype=np.uint8)
            )
            img.save(cats_folder / f"cat_{i}.png")
            img.save(dogs_folder / f"dog_{i}.png")

        preprocessor = ClassificationImageTrainingPreprocessing(
            training_folder_images=training_folder, folder_path=self.temp_dir
        )
        preprocessor.get_classes_mapping()
        invalid_images, invalid_labels = [], []
        images, labels = preprocessor.data_preparing(
            training_folder,
            invalid_images,
            invalid_labels,
            preprocessor.training_class,
        )
        assert "cats" in preprocessor.training_class
        assert "dogs" in preprocessor.training_class
        assert len(invalid_labels) == 10
        assert len(invalid_images) == 10
        assert len(images) == 10
        assert len(labels) == 10

    def test_data_overview(self, tmp_path):
        training_folder = tmp_path / "training_folder"
        training_folder.mkdir()
        cats_folder = training_folder / "cats"
        cats_folder.mkdir()
        dogs_folder = training_folder / "dogs"
        dogs_folder.mkdir()
        for i in range(5):
            img = Image.fromarray(
                np.random.randint(0, 256, (225, 225, 3), dtype=np.uint8)
            )
            img.save(cats_folder / f"cat_{i}.png")
            img.save(dogs_folder / f"dog_{i}.png")

        preprocessor = ClassificationImageTrainingPreprocessing(
            training_folder_images=training_folder, folder_path=self.temp_dir
        )
        preprocessor.common_preprocessing()
        assert "training_folder" in preprocessor.report_data["data_overview"]
        assert "validation_folder" not in preprocessor.report_data["data_overview"]
        assert (
            preprocessor.report_data["data_overview"]["training_folder"][
                "images_len"
            ]
            == 10
        )
        assert (
            preprocessor.report_data["data_overview"]["training_folder"][
                "invalid_len"
            ]
            == 0
        )

    def test_splitting(self, tmp_path):
        training_folder = tmp_path / "training_folder"
        training_folder.mkdir()
        cats_folder = training_folder / "cats"
        cats_folder.mkdir()
        dogs_folder = training_folder / "dogs"
        dogs_folder.mkdir()
        for i in range(5):
            img = Image.fromarray(
                np.random.randint(0, 256, (225, 225, 3), dtype=np.uint8)
            )
            img.save(cats_folder / f"cat_{i}.png")
            img.save(dogs_folder / f"dog_{i}.png")

        preprocessor = ClassificationImageTrainingPreprocessing(
            training_folder_images=training_folder, folder_path=self.temp_dir
        )
        preprocessor.common_preprocessing()
        assert len(preprocessor.training_paths) == 10
        assert len(preprocessor.training_labels) == 10
        assert preprocessor.validation_paths is None
        assert preprocessor.validation_labels is None
        preprocessor = ClassificationImageTrainingPreprocessing(
            training_folder_images=training_folder,
            folder_path=self.temp_dir,
            split_training=True,
        )
        preprocessor.common_preprocessing()
        assert len(preprocessor.training_paths) == 8
        assert len(preprocessor.training_labels) == 8
        assert len(preprocessor.validation_paths) == 2
        assert len(preprocessor.validation_labels) == 2

    def test_get_loaders(self, tmp_path):
        training_folder = tmp_path / "training_folder"
        training_folder.mkdir()
        cats_folder = training_folder / "cats"
        cats_folder.mkdir()
        dogs_folder = training_folder / "dogs"
        dogs_folder.mkdir()
        for i in range(5):
            img = Image.fromarray(
                np.random.randint(0, 256, (225, 225, 3), dtype=np.uint8)
            )
            img.save(cats_folder / f"cat_{i}.png")
            img.save(dogs_folder / f"dog_{i}.png")

        preprocessor = ClassificationImageTrainingPreprocessing(
            training_folder_images=training_folder, folder_path=self.temp_dir
        )
        preprocessor.common_preprocessing()
        assert preprocessor.training_loader is not None
        assert preprocessor.validation_loader is None
        preprocessor = ClassificationImageTrainingPreprocessing(
            training_folder_images=training_folder,
            folder_path=self.temp_dir,
            split_training=True,
        )
        preprocessor.common_preprocessing()
        assert preprocessor.training_loader is not None
        assert preprocessor.validation_loader is not None



import os
from collections import Counter

import matplotlib.pyplot as plt
import seaborn as sns

from accelera.src.automl.wrappers.tabular_graph_base import TabularGraphBase


class TextGraph(TabularGraphBase):
    def __init__(self, df, col_name, target_name, folder_path):
        super().__init__(df, col_name, target_name, folder_path)
        words = self.graph_df[col_name].str.split().explode()
        top_7_words = Counter(words).most_common(7)
        self.word, self.count = zip(*top_7_words)

    def build_graph(self):
        _, ax = plt.subplots(1, 2, figsize=(12, 4))
        ax[0].pie(
            [float(self.nulls_percent), float(100 - self.nulls_percent)],
            labels=["Nulls", "Not Nulls"],
            autopct="%1.1f%%",
            colors=["#021D25", "#ADD8E6"],
        )
        self.graph_df = self.graph_df[[self.col_name, self.target_name]].dropna()
        sns.barplot(x=list(self.word), y=list(self.count), ax=ax[1])
        ax[1].set_title(f"Top 7 words in {self.col_name}")
        ax[1].set_xlabel("Words")
        ax[1].set_ylabel("Count")

        plt.tight_layout()
        plt.savefig(os.path.join(self.folder_path, f"{self.col_name}.png"))
        plt.close()



from unittest.mock import patch

import numpy as np
import pytest
from PIL import Image

from accelera.src.automl.core.image_dataset import ImageDataset


class TestImageDataset:
    @pytest.fixture(autouse=True)
    def create_dummy_image(self):
        self.img = np.random.randint(0, 256, (125, 125, 3), dtype=np.uint8)
        self.img = Image.fromarray(self.img)

    def test_length(self):
        paths = ["imag1", "imag2", "imag3"]
        dataset = ImageDataset(image_paths=paths)
        assert len(dataset) == 3

    def test_random_brightness(self):
        dataset = ImageDataset(image_paths=[], brightness=False)
        result_img = dataset.random_brightness(self.img)
        assert np.array_equal(np.array(self.img), np.array(result_img))
        dataset = ImageDataset(
            image_paths=[], brightness=True, augmentation_probability=0
        )
        result_img = dataset.random_brightness(self.img)
        assert np.array_equal(np.array(self.img), np.array(result_img))
        dataset = ImageDataset(
            image_paths=[],
            brightness=True,
            augmentation_probability=1,
            brightness_factors=(0.5, 0.5),
        )
        with patch("random.random", return_value=0.0):
            result_img = dataset.random_brightness(self.img)
        assert not np.array_equal(np.array(self.img), np.array(result_img))

    def test_random_contrast(self):
        dataset = ImageDataset(image_paths=[], contrast=False)
        result_img = dataset.random_contrast(self.img)
        assert np.array_equal(np.array(self.img), np.array(result_img))
        dataset = ImageDataset(
            image_paths=[], contrast=True, augmentation_probability=0
        )
        result_img = dataset.random_contrast(self.img)
        assert np.array_equal(np.array(self.img), np.array(result_img))
        dataset = ImageDataset(
            image_paths=[],
            contrast=True,
            augmentation_probability=1,
            contrast_factors=(0.5, 0.5),
        )
        with patch("random.random", return_value=0.0):
            result_img = dataset.random_contrast(self.img)
        assert not np.array_equal(np.array(self.img), np.array(result_img))



import sys
from pathlib import Path
from types import SimpleNamespace

import pandas as pd
import pytest

from accelera.src.utils.dataset_retriever import DatasetRetriever


class TestDatasetRetriever:
    @pytest.fixture
    def connected_retriever(self, tmp_path):
        retriever = DatasetRetriever()
        retriever.is_connected = True
        retriever.cache_dir = str(tmp_path)
        return retriever

    @pytest.fixture
    def fake_gdown(self, monkeypatch):
        calls = []

        def download(**kwargs):
            calls.append(kwargs)
            Path(kwargs["output"]).write_text("feature,target\n1,0\n")

        monkeypatch.setitem(
            sys.modules,
            "gdown",
            SimpleNamespace(download=download),
        )
        monkeypatch.setattr(
            "accelera.src.utils.dataset_retriever.print_msg",
            lambda *args, **kwargs: None,
        )
        return calls

    def test_available_datasets_extracts_unique_names(self, monkeypatch):
        retriever = DatasetRetriever()
        monkeypatch.setattr(
            retriever,
            "_fetch_url_content",
            lambda url: {
                "content": (
                    "<a>customer_purchase_data.csv</a>"
                    "<a>Housing.csv</a>"
                    "<a>Housing.csv</a>"
                    "<a>Samplex22.csv</a>"
                )
            },
        )

        result = retriever.available_datasets()

        assert set(result) == {
            "customer_purchase_data",
            "Housing",
            "Sample",
        }

    def test_connect_and_close_manage_cache_dir(self, monkeypatch, tmp_path):
        cache_dir = tmp_path / "accelera-cache"
        monkeypatch.setattr(
            "accelera.src.utils.dataset_retriever.user_cache_dir",
            lambda _: str(cache_dir),
        )

        retriever = DatasetRetriever()
        retriever.connect()

        assert retriever.is_connected is True
        assert retriever.cache_dir == str(cache_dir)
        assert cache_dir.exists()

        retriever.close()

        assert retriever.is_connected is False
        assert retriever.cache_dir is None
        assert not cache_dir.exists()

    def test_retrieve_dataset_requires_connect(self):
        retriever = DatasetRetriever()

        with pytest.raises(RuntimeError, match="Call `connect\\(\\)` first"):
            retriever.retrieve_dataset("customer_purchase_data")

    def test_retrieve_dataset_returns_cached_dataframe(
        self, connected_retriever, fake_gdown
    ):
        dataset_path = Path(connected_retriever.cache_dir) / "cached.csv"
        dataset_path.write_text("feature,target\n1,0\n2,1\n")

        result = connected_retriever.retrieve_dataset("cached", df=True)

        expected = pd.DataFrame({"feature": [1, 2], "target": [0, 1]})
        pd.testing.assert_frame_equal(result, expected)
        assert fake_gdown == []

    def test_retrieve_dataset_downloads_known_dataset(
        self, connected_retriever, fake_gdown, monkeypatch
    ):
        monkeypatch.setitem(
            connected_retriever.retrieve_dataset.__globals__["config"].DATASETS,
            "toy_dataset",
            {"id": "file-123", "metadata": {}},
        )

        result = connected_retriever.retrieve_dataset("toy_dataset")

        assert result == str(Path(connected_retriever.cache_dir) / "toy_dataset.csv")
        assert fake_gdown == [
            {
                "id": "file-123",
                "output": result,
                "quiet": False,
            }
        ]



import os

import matplotlib.pyplot as plt
import seaborn as sns

from accelera.src.automl.wrappers.tabular_graph_base import TabularGraphBase


class CategoricalClassification(TabularGraphBase):
    def __init__(self, df, col_name, target_name, folder_path):
        super().__init__(df, col_name, target_name, folder_path)
        if self.graph_df[col_name].nunique() > 5:
            top_5_categories = self.graph_df[col_name].value_counts().nlargest(5)
            self.graph_df[col_name] = self.graph_df[col_name].where(
                self.graph_df[col_name].isin(top_5_categories.index),
                other="Other",
            )
            self.is_top5_applied = True
        else:
            self.is_top5_applied = False

    def build_graph(self):
        _, ax = plt.subplots(1, 3, figsize=(12, 4))
        ax[0].pie(
            [float(self.nulls_percent), float(100 - self.nulls_percent)],
            labels=["Nulls", "Not Nulls"],
            autopct="%1.1f%%",
            colors=["#021D25", "#ADD8E6"],
        )
        ax[0].set_title(f"{self.col_name} Null percentage")
        sns.countplot(data=self.graph_df, x=self.col_name, ax=ax[1])
        if self.is_top5_applied:
            ax[1].set_title(
                f"{self.col_name} Distribution\n (Top 5 Categories + Other)"
            )
            title = (
                f"{self.col_name} vs {self.target_name}\n Distribution\n"
                f"(Top 5 Categories + Other) "
            )
            ax[2].set_title(title)
        else:
            ax[1].set_title(f"{self.col_name} Distribution")
            ax[2].set_title(f"{self.col_name} vs {self.target_name}\n Distribution")
        ax[1].set_xlabel(self.col_name)
        ax[1].set_ylabel("Count")
        for label in ax[1].get_xticklabels():
            label.set_rotation(45)
            label.set_horizontalalignment("right")
        self.graph_df = self.graph_df[[self.col_name, self.target_name]].dropna()
        sns.countplot(
            data=self.graph_df, x=self.col_name, hue=self.target_name, ax=ax[2]
        )
        ax[2].set_ylabel(self.col_name)
        ax[2].set_xlabel(self.target_name)
        for label in ax[2].get_xticklabels():
            label.set_rotation(45)
            label.set_horizontalalignment("right")
        plt.tight_layout()
        plt.savefig(os.path.join(self.folder_path, f"{self.col_name}.png"))
        plt.close()



# Make the core classes available at the package level
import os

from accelera.src.config import config

# Add the API modules to ensure the public API is accessible at this level
try:
    from accelera.api import *  # noqa: F403
except ImportError:
    pass



import sys
from setuptools import setup

if sys.platform != "linux":
    # Refuse to install from the source distribution if it's not linux.
    # When you want to support windows later, just delete this `setup.py` file 
    # or remove this check.
    import warnings
    warnings.warn(
        "\n\n*** ACCELERA INSTALLATION ERROR ***\n"
        "This version of accelera is only supported on Linux. "
        "Windows support will be added in a future release.\n"
        "***********************************\n"
    )
    sys.exit(1)

setup()



from accelera.src.automl.core.testing_tabular_preprocessing_base import (
    TestingTabularPreprocessingBase,
)
from accelera.src.utils.preprocessing import check_path_exists
from accelera.src.utils.preprocessing import load_pickle
from accelera.src.utils.preprocessing import lower_data


class TextTestingPreprocessing(TestingTabularPreprocessingBase):
    def __init__(self, df, folder_path=None):
        super().__init__(df, folder_path=folder_path)
        check_path_exists(self.folder_path, "data_info.pkl")
        self.data_info = load_pickle(self.folder_path, "data_info.pkl")
        self.target_col = self.data_info["target_col"]
        self.text_col = self.data_info["text_col"]
        self.target_mode = self.data_info["target_mode"]
        if self.text_col == self.target_col:
            raise ValueError("target column and text column must not be the same")
        if self.text_col not in self.df.columns:
            raise ValueError(f"data dose not has this text col {self.text_col}")
        if self.target_col not in self.df.columns:
            self.features_only = True

    def feature_preprocessing(self):
        try:
            X_test = self.df[self.text_col].fillna("")
            X_test = self.training_preprocessor.transform(X_test)
            return X_test
        except Exception as e:
            raise ValueError(f"Error in target preprocessing: {e}")

    def target_preprocessing(self):
        try:
            if self.features_only:
                return None
            else:
                y_test = self.df[self.target_col].fillna(self.target_mode)
                y_test = self.target_preprocessor.transform(y_test)
            return y_test
        except Exception as e:
            raise ValueError(f"Error in target preprocessing: {e}")

    def common_preprocessing(self):
        lower_data(self.df)
        X_test = self.feature_preprocessing()
        y_test = self.target_preprocessing()
        return X_test, y_test



from accelera.src.accelera_pipe.core.metric import BaseMetric
from accelera.src.utils.array_utils import convert_to_array
from accelera.src.utils.array_utils import validate_array_shape


class SupervisedMetric(BaseMetric):
    def __init__(
        self,
        metric_name,
        metric,
        y_true=None,
        plot_func=None,
        labels_name=None,
        headers_name=None,
        **params,
    ):
        super().__init__(
            metric_name,
            metric,
            y_true=y_true,
            X=None,
            plot_func=plot_func,
            labels_name=labels_name,
            headers_name=headers_name,
            **params,
        )

    def set_y_true(self, new_y_true):
        self.y_true = convert_to_array(new_y_true)

    def execute(self, y_pred):
        y_pred = convert_to_array(y_pred)
        if self.y_true is None:
            raise ValueError("y_true must be provided for supervised metrics.")
        validate_array_shape(y_pred, self.y_true.shape[0], ["y_pred", "y_true"])
        result = self.metric(self.y_true, y_pred, **self.params)
        output = {
            "metric name": self.metric_name,
            "result": result,
            "plot_func": self.plot_func,
            "labels_name": self.labels_name,
            "headers_name": self.headers_name,
        }
        return output



import os

import pandas as pd
from sklearn.model_selection import train_test_split

from accelera.src.automl.core.preprocessing_base import PreprocessingBase
from accelera.src.utils.preprocessing import check_path_exists


class ImageTrainingPreprocessing(PreprocessingBase):
    def __init__(
        self,
        training_folder,
        folder_path,
        validation_folder,
        split_training,
        val_size,
        random_state,
        images_size,
        augment=True,
        batch_size=16,
        augmentation_probability=0.5,
        horizontal_flip=True,
        vertical_flip=True,
        rotation=True,
        rotation_angle=30,
        brightness=True,
        brightness_factors=(0.8, 1.2),
        contrast=True,
        contrast_factors=(0.8, 1.2),
    ):
        super().__init__(folder_path)
        self.training_folder = training_folder
        self.validation_folder = validation_folder
        self.split_training = split_training
        self.val_size = val_size
        self.random_state = random_state
        self.image_size = images_size
        self.augment = augment
        self.batch_size = batch_size
        self.horizontal_flip = horizontal_flip
        self.vertical_flip = vertical_flip
        self.augmentation_probability = augmentation_probability
        self.rotation = rotation
        self.rotation_angle = rotation_angle
        self.brightness = brightness
        self.brightness_factors = brightness_factors
        self.contrast = contrast
        self.contrast_factors = contrast_factors
        self.report_data = {}
        if self.training_folder is None:
            raise ValueError("Training folder must be not null")
        check_path_exists(self.training_folder, "")
        if self.validation_folder is not None:
            check_path_exists(self.validation_folder, "")
            if self.split_training:
                raise ValueError(
                    "The validation folder not none so there is no need "
                    "for splitting data again to training and validation"
                )
            if self.training_folder == self.validation_folder:
                raise ValueError(
                    "The validation folder and training folder must be different"
                )

        if (not (isinstance(self.val_size, (float)))) or (
            not (0 < self.val_size <= 0.5)
        ):
            raise ValueError("Test size is invalid it must be less than 0.5")
        if (self.random_state is not None) and not (
            isinstance(self.random_state, int)
        ):
            raise ValueError("Random state is invalid it must be integer or None")
        if not isinstance(self.image_size, tuple):
            raise ValueError("Image size must be tuple")
        if not isinstance(self.image_size[0], int) or not isinstance(
            self.image_size[1], int
        ):
            raise ValueError("Image size is not integer")
        if not (32 <= self.image_size[0] <= 1024) or not (
            32 <= self.image_size[1] <= 1024
        ):
            raise ValueError(
                "Image size is must be greater than or equal 32 and "
                "less than or equal 1024"
            )
        if not isinstance(self.image_size, tuple):
            raise ValueError("Image size must be tuple")
        if not isinstance(self.image_size[0], int) or not isinstance(
            self.image_size[1], int
        ):
            raise ValueError("Image size is not integer")
        if not (32 <= self.image_size[0] <= 1024) or not (
            32 <= self.image_size[1] <= 1024
        ):
            raise ValueError(
                "Image size is must be greater than or equal 32 and "
                "less than or equal 1024"
            )
        if not isinstance(self.batch_size, int):
            raise ValueError("batch_size must be a integer")

        if self.batch_size <= 0:
            raise ValueError("batch_size must be a positive value")

        if not isinstance(self.augment, bool):
            raise ValueError("augment must be a boolean")

        if not isinstance(self.horizontal_flip, bool):
            raise ValueError("horizontal_flip must be a boolean")

        if not isinstance(self.vertical_flip, bool):
            raise ValueError("vertical_flip must be a boolean")

        if not isinstance(self.rotation, bool):
            raise ValueError("rotation must be a boolean")

        if not isinstance(self.brightness, bool):
            raise ValueError("brightness must be a boolean")

        if not isinstance(self.contrast, bool):
            raise ValueError("contrast must be a boolean")
        if not isinstance(self.augmentation_probability, (int, float)) or not (
            0 <= self.augmentation_probability <= 1
        ):
            raise ValueError("augmentation_probability must be between [0,1]")
        if (
            not isinstance(self.rotation_angle, (int, float))
            or self.rotation_angle < 0
        ):
            raise ValueError("rotation_angle must be positive integer or float")

        if (
            not isinstance(self.brightness_factors, tuple)
            or len(self.brightness_factors) != 2
            or not all(isinstance(x, (int, float)) for x in self.brightness_factors)
        ):
            raise ValueError(
                "brightness_factors must be tuple of two items float or integers"
            )
        if (
            not isinstance(self.contrast_factors, tuple)
            or len(self.contrast_factors) != 2
            or not all(isinstance(x, (int, float)) for x in self.contrast_factors)
        ):
            raise ValueError(
                "contrast_factors must be tuple of two items float or integers"
            )
        os.makedirs(self.folder_path, exist_ok=True)

    def get_sample_random(self, data_type, images_path, labels, num_samples=5):
        df = (
            pd.DataFrame(
                {
                    f"{data_type}_paths": images_path,
                    f"{data_type}_labels": labels,
                }
            )
            .sample(frac=1, random_state=self.random_state)
            .reset_index(drop=True)
        ).head(num_samples)
        return df

    def splitting(
        self,
        training_folder_images_paths,
        validation_folder_images_paths,
        training_folder_images_labels,
        validation_folder_images_labels,
    ):
        if self.split_training:
            (
                self.training_paths,
                self.validation_paths,
                self.training_labels,
                self.validation_labels,
            ) = train_test_split(
                training_folder_images_paths,
                training_folder_images_labels,
                test_size=self.val_size,
                random_state=self.random_state,
            )
            self.report_data["split_data"] = {
                "validation_size": self.val_size,
                "training_data_size": len(self.training_paths),
                "random_training_sample": self.get_sample_random(
                    "training", self.training_paths, self.training_labels
                ),
                "validation_data_size": len(self.validation_paths),
                "random_validation_sample": self.get_sample_random(
                    "validation", self.validation_paths, self.validation_labels
                ),
            }
        else:
            (
                self.training_paths,
                self.validation_paths,
                self.training_labels,
                self.validation_labels,
            ) = (
                training_folder_images_paths,
                validation_folder_images_paths,
                training_folder_images_labels,
                validation_folder_images_labels,
            )
